@@ -1,5 +1,5 @@
 /**
- * \file <alsa/error.h>
+ * \file include/error.h
  * \brief Application interface library for the ALSA driver
  * \author Jaroslav Kysela <perex@suse.cz>
  * \author Abramo Bagnara <abramo@alsa-project.org>
@@ -40,6 +40,7 @@ extern "C" {
 
 #define SND_ERROR_BEGIN				500000			/**< Lower boundary of sound error codes. */
 #define SND_ERROR_INCOMPATIBLE_VERSION		(SND_ERROR_BEGIN+0)	/**< Kernel/library protocols are not compatible. */
+#define SND_ERROR_ALISP_NIL			(SND_ERROR_BEGIN+1)	/**< Lisp encountered an error during acall. */
 
 const char *snd_strerror(int errnum);
 
