@@ -1,5 +1,5 @@
 /**
- * \file <alsa/pcm_plugin.h>
+ * \file include/pcm_plugin.h
  * \brief Common PCM plugin code
  * \author Abramo Bagnara <abramo@alsa-project.org>
  * \author Jaroslav Kysela <perex@suse.cz>
@@ -11,8 +11,8 @@
  * \warning Using of contents of this header file might be dangerous
  *	    in the sense of compatibility reasons. The contents might be
  *	    freely changed in future.
- *
- *
+ */
+/*
  *   This library is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Lesser General Public License as
  *   published by the Free Software Foundation; either version 2.1 of
@@ -68,7 +68,7 @@ typedef int snd_pcm_route_ttable_entry_t;
 int snd_pcm_hw_open(snd_pcm_t **pcmp, const char *name,
 		    int card, int device, int subdevice,
 		    snd_pcm_stream_t stream, int mode,
-		    int mmap_emulation);
+		    int mmap_emulation, int sync_ptr_ioctl);
 int _snd_pcm_hw_open(snd_pcm_t **pcmp, const char *name,
 		     snd_config_t *root ATTRIBUTE_UNUSED, snd_config_t *conf,
 		     snd_pcm_stream_t stream, int mode);
