@@ -92,7 +92,7 @@ static int snd_pcm_copy_hw_refine(snd_pcm_t *pcm, snd_pcm_hw_params_t *params)
 				       snd_pcm_copy_hw_refine_cchange,
 				       snd_pcm_copy_hw_refine_sprepare,
 				       snd_pcm_copy_hw_refine_schange,
-				       snd_pcm_plugin_hw_refine_slave);
+				       snd_pcm_generic_hw_refine);
 }
 
 static int snd_pcm_copy_hw_params(snd_pcm_t *pcm, snd_pcm_hw_params_t *params)
@@ -101,7 +101,7 @@ static int snd_pcm_copy_hw_params(snd_pcm_t *pcm, snd_pcm_hw_params_t *params)
 				       snd_pcm_copy_hw_refine_cchange,
 				       snd_pcm_copy_hw_refine_sprepare,
 				       snd_pcm_copy_hw_refine_schange,
-				       snd_pcm_plugin_hw_params_slave);
+				       snd_pcm_generic_hw_params);
 }
 
 static snd_pcm_uframes_t
@@ -149,23 +149,22 @@ static void snd_pcm_copy_dump(snd_pcm_t *pcm, snd_output_t *out)
 		snd_pcm_dump_setup(pcm, out);
 	}
 	snd_output_printf(out, "Slave: ");
-	snd_pcm_dump(copy->plug.slave, out);
+	snd_pcm_dump(copy->plug.gen.slave, out);
 }
 
 static snd_pcm_ops_t snd_pcm_copy_ops = {
-	.close = snd_pcm_plugin_close,
-	.info = snd_pcm_plugin_info,
+	.close = snd_pcm_generic_close,
+	.info = snd_pcm_generic_info,
 	.hw_refine = snd_pcm_copy_hw_refine,
 	.hw_params = snd_pcm_copy_hw_params,
-	.hw_free = snd_pcm_plugin_hw_free,
-	.sw_params = snd_pcm_plugin_sw_params,
-	.channel_info = snd_pcm_plugin_channel_info,
+	.hw_free = snd_pcm_generic_hw_free,
+	.sw_params = snd_pcm_generic_sw_params,
+	.channel_info = snd_pcm_generic_channel_info,
 	.dump = snd_pcm_copy_dump,
-	.nonblock = snd_pcm_plugin_nonblock,
-	.async = snd_pcm_plugin_async,
-	.poll_revents = snd_pcm_plugin_poll_revents,
-	.mmap = snd_pcm_plugin_mmap,
-	.munmap = snd_pcm_plugin_munmap,
+	.nonblock = snd_pcm_generic_nonblock,
+	.async = snd_pcm_generic_async,
+	.mmap = snd_pcm_generic_mmap,
+	.munmap = snd_pcm_generic_munmap,
 };
 
 /**
@@ -194,8 +193,8 @@ int snd_pcm_copy_open(snd_pcm_t **pcmp, const char *name, snd_pcm_t *slave, int
 	copy->plug.write = snd_pcm_copy_write_areas;
 	copy->plug.undo_read = snd_pcm_plugin_undo_read_generic;
 	copy->plug.undo_write = snd_pcm_plugin_undo_write_generic;
-	copy->plug.slave = slave;
-	copy->plug.close_slave = close_slave;
+	copy->plug.gen.slave = slave;
+	copy->plug.gen.close_slave = close_slave;
 
 	err = snd_pcm_new(&pcm, SND_PCM_TYPE_COPY, name, slave->stream, slave->mode);
 	if (err < 0) {
