@@ -49,6 +49,11 @@
 const char *_snd_module_pcm_dshare = "";
 #endif
 
+#ifndef DOC_HIDDEN
+/* start is pending - this state happens when rate plugin does a delayed commit */
+#define STATE_RUN_PENDING	1024
+#endif
+
 static void do_silence(snd_pcm_t *pcm)
 {
 	snd_pcm_direct_t *dshare = pcm->private_data;
@@ -94,30 +99,51 @@ static void share_areas(snd_pcm_direct_t *dshare,
 /*
  *  synchronize shm ring buffer with hardware
  */
-static void snd_pcm_dshare_sync_area(snd_pcm_t *pcm, snd_pcm_uframes_t size)
+static void snd_pcm_dshare_sync_area(snd_pcm_t *pcm)
 {
 	snd_pcm_direct_t *dshare = pcm->private_data;
-	snd_pcm_uframes_t appl_ptr, slave_appl_ptr, transfer;
+	snd_pcm_uframes_t appl_ptr, slave_appl_ptr, slave_bsize;
+	snd_pcm_uframes_t size, slave_hw_ptr;
 	const snd_pcm_channel_area_t *src_areas, *dst_areas;
 	
-	/* get the start of update area */
-	appl_ptr = dshare->appl_ptr - size;
-	if (appl_ptr > pcm->boundary)
-		appl_ptr += pcm->boundary;
-	appl_ptr %= pcm->buffer_size;
+	/* calculate the size to transfer */
+	size = dshare->appl_ptr - dshare->last_appl_ptr;
+	if (! size)
+		return;
+	slave_bsize = dshare->shmptr->s.buffer_size;
+	slave_hw_ptr = dshare->slave_hw_ptr;
+	/* don't write on the last active period - this area may be cleared
+	 * by the driver during write operation...
+	 */
+	slave_hw_ptr -= slave_hw_ptr % dshare->shmptr->s.period_size;
+	slave_hw_ptr += slave_bsize;
+	if (dshare->slave_hw_ptr > dshare->slave_appl_ptr)
+		slave_hw_ptr -= dshare->shmptr->s.boundary;
+	if (dshare->slave_appl_ptr + size >= slave_hw_ptr)
+		size = slave_hw_ptr - dshare->slave_appl_ptr;
+	if (! size)
+		return;
 	/* add sample areas here */
 	src_areas = snd_pcm_mmap_areas(pcm);
 	dst_areas = snd_pcm_mmap_areas(dshare->spcm);
-	slave_appl_ptr = dshare->slave_appl_ptr % dshare->shmptr->s.buffer_size;
+	appl_ptr = dshare->last_appl_ptr % pcm->buffer_size;
+	dshare->last_appl_ptr += size;
+	dshare->last_appl_ptr %= pcm->boundary;
+	slave_appl_ptr = dshare->slave_appl_ptr % slave_bsize;
 	dshare->slave_appl_ptr += size;
 	dshare->slave_appl_ptr %= dshare->shmptr->s.boundary;
-	while (size > 0) {
-		transfer = appl_ptr + size > pcm->buffer_size ? pcm->buffer_size - appl_ptr : size;
-		transfer = slave_appl_ptr + transfer > dshare->shmptr->s.buffer_size ? dshare->shmptr->s.buffer_size - slave_appl_ptr : transfer;
-		size -= transfer;
+	for (;;) {
+		snd_pcm_uframes_t transfer = size;
+		if (appl_ptr + transfer > pcm->buffer_size)
+			transfer = pcm->buffer_size - appl_ptr;
+		if (slave_appl_ptr + transfer > slave_bsize)
+			transfer = slave_bsize - slave_appl_ptr;
 		share_areas(dshare, src_areas, dst_areas, appl_ptr, slave_appl_ptr, transfer);
+		size -= transfer;
+		if (! size)
+			break;
 		slave_appl_ptr += transfer;
-		slave_appl_ptr %= dshare->shmptr->s.buffer_size;
+		slave_appl_ptr %= slave_bsize;
 		appl_ptr += transfer;
 		appl_ptr %= pcm->buffer_size;
 	}
@@ -146,6 +172,10 @@ static int snd_pcm_dshare_sync_ptr(snd_pcm_t *pcm)
 	diff = slave_hw_ptr - old_slave_hw_ptr;
 	if (diff == 0)		/* fast path */
 		return 0;
+	if (dshare->state != SND_PCM_STATE_RUNNING &&
+	    dshare->state != SND_PCM_STATE_DRAINING)
+		/* not really started yet - don't update hw_ptr */
+		return 0;
 	if (diff < 0) {
 		slave_hw_ptr += dshare->shmptr->s.boundary;
 		diff = slave_hw_ptr - old_slave_hw_ptr;
@@ -155,17 +185,23 @@ static int snd_pcm_dshare_sync_ptr(snd_pcm_t *pcm)
 	// printf("sync ptr diff = %li\n", diff);
 	if (pcm->stop_threshold >= pcm->boundary)	/* don't care */
 		return 0;
-	if ((avail = snd_pcm_mmap_playback_avail(pcm)) >= pcm->stop_threshold) {
+	avail = snd_pcm_mmap_playback_avail(pcm);
+	if (avail > dshare->avail_max)
+		dshare->avail_max = avail;
+	if (avail >= pcm->stop_threshold) {
 		struct timeval tv;
+		snd_timer_stop(dshare->timer);
 		gettimeofday(&tv, 0);
 		dshare->trigger_tstamp.tv_sec = tv.tv_sec;
 		dshare->trigger_tstamp.tv_nsec = tv.tv_usec * 1000L;
-		dshare->state = SND_PCM_STATE_XRUN;
-		dshare->avail_max = avail;
-		return -EPIPE;
+		if (dshare->state == SND_PCM_STATE_RUNNING) {
+			dshare->state = SND_PCM_STATE_XRUN;
+			return -EPIPE;
+		}
+		dshare->state = SND_PCM_STATE_SETUP;
+		/* clear queue to remove pending poll events */
+		snd_pcm_direct_clear_timer_queue(dshare);
 	}
-	if (avail > dshare->avail_max)
-		dshare->avail_max = avail;
 	return 0;
 }
 
@@ -176,7 +212,6 @@ static int snd_pcm_dshare_sync_ptr(snd_pcm_t *pcm)
 static int snd_pcm_dshare_status(snd_pcm_t *pcm, snd_pcm_status_t * status)
 {
 	snd_pcm_direct_t *dshare = pcm->private_data;
-	snd_pcm_state_t state;
 
 	switch (dshare->state) {
 	case SNDRV_PCM_STATE_DRAINING:
@@ -187,8 +222,7 @@ static int snd_pcm_dshare_status(snd_pcm_t *pcm, snd_pcm_status_t * status)
 		break;
 	}
 	memset(status, 0, sizeof(*status));
-	state = snd_pcm_state(dshare->spcm);
-	status->state = state == SND_PCM_STATE_RUNNING ? dshare->state : state;
+	status->state = snd_pcm_state(dshare->spcm);
 	status->trigger_tstamp = dshare->trigger_tstamp;
 	status->tstamp = snd_pcm_hw_fast_tstamp(dshare->spcm);
 	status->avail = snd_pcm_mmap_playback_avail(pcm);
@@ -202,13 +236,14 @@ static snd_pcm_state_t snd_pcm_dshare_state(snd_pcm_t *pcm)
 	snd_pcm_direct_t *dshare = pcm->private_data;
 	switch (snd_pcm_state(dshare->spcm)) {
 	case SND_PCM_STATE_SUSPENDED:
-		return -ESTRPIPE;
+		return SND_PCM_STATE_SUSPENDED;
 	case SND_PCM_STATE_DISCONNECTED:
-		dshare->state = SNDRV_PCM_STATE_DISCONNECTED;
-		return -ENOTTY;
+		return SND_PCM_STATE_DISCONNECTED;
 	default:
 		break;
 	}
+	if (dshare->state == STATE_RUN_PENDING)
+		return SNDRV_PCM_STATE_RUNNING;
 	return dshare->state;
 }
 
@@ -223,8 +258,10 @@ static int snd_pcm_dshare_delay(snd_pcm_t *pcm, snd_pcm_sframes_t *delayp)
 		err = snd_pcm_dshare_sync_ptr(pcm);
 		if (err < 0)
 			return err;
+		/* fallthru */
 	case SNDRV_PCM_STATE_PREPARED:
 	case SNDRV_PCM_STATE_SUSPENDED:
+	case STATE_RUN_PENDING:
 		*delayp = snd_pcm_mmap_playback_hw_avail(pcm);
 		return 0;
 	case SNDRV_PCM_STATE_XRUN:
@@ -263,20 +300,33 @@ static int snd_pcm_dshare_prepare(snd_pcm_t *pcm)
 	snd_pcm_direct_check_interleave(dshare, pcm);
 	// assert(pcm->boundary == dshare->shmptr->s.boundary);	/* for sure */
 	dshare->state = SND_PCM_STATE_PREPARED;
-	dshare->appl_ptr = 0;
+	dshare->appl_ptr = dshare->last_appl_ptr = 0;
 	dshare->hw_ptr = 0;
-	return 0;
+	return snd_pcm_direct_set_timer_params(dshare);
 }
 
 static int snd_pcm_dshare_reset(snd_pcm_t *pcm)
 {
 	snd_pcm_direct_t *dshare = pcm->private_data;
 	dshare->hw_ptr %= pcm->period_size;
-	dshare->appl_ptr = dshare->hw_ptr;
+	dshare->appl_ptr = dshare->last_appl_ptr = dshare->hw_ptr;
 	dshare->slave_appl_ptr = dshare->slave_hw_ptr = *dshare->spcm->hw.ptr;
 	return 0;
 }
 
+static int snd_pcm_dshare_start_timer(snd_pcm_direct_t *dshare)
+{
+	int err;
+
+	snd_pcm_hwsync(dshare->spcm);
+	dshare->slave_appl_ptr = dshare->slave_hw_ptr = *dshare->spcm->hw.ptr;
+	err = snd_timer_start(dshare->timer);
+	if (err < 0)
+		return err;
+	dshare->state = SND_PCM_STATE_RUNNING;
+	return 0;
+}
+
 static int snd_pcm_dshare_start(snd_pcm_t *pcm)
 {
 	snd_pcm_direct_t *dshare = pcm->private_data;
@@ -286,18 +336,16 @@ static int snd_pcm_dshare_start(snd_pcm_t *pcm)
 	
 	if (dshare->state != SND_PCM_STATE_PREPARED)
 		return -EBADFD;
-	err = snd_timer_start(dshare->timer);
-	if (err < 0)
-		return err;
-	dshare->state = SND_PCM_STATE_RUNNING;
-	snd_pcm_hwsync(dshare->spcm);
-	dshare->slave_appl_ptr = dshare->slave_hw_ptr = *dshare->spcm->hw.ptr;
 	avail = snd_pcm_mmap_playback_hw_avail(pcm);
-	if (avail < 0)
+	if (avail == 0)
+		dshare->state = STATE_RUN_PENDING;
+	else if (avail < 0)
 		return 0;
-	if (avail > (snd_pcm_sframes_t)pcm->buffer_size)
-		avail = pcm->buffer_size;
-	snd_pcm_dshare_sync_area(pcm, avail);
+	else {
+		if ((err = snd_pcm_dshare_start_timer(dshare)) < 0)
+			return err;
+		snd_pcm_dshare_sync_area(pcm);
+	}
 	gettimeofday(&tv, 0);
 	dshare->trigger_tstamp.tv_sec = tv.tv_sec;
 	dshare->trigger_tstamp.tv_nsec = tv.tv_usec * 1000L;
@@ -309,7 +357,7 @@ static int snd_pcm_dshare_drop(snd_pcm_t *pcm)
 	snd_pcm_direct_t *dshare = pcm->private_data;
 	if (dshare->state == SND_PCM_STATE_OPEN)
 		return -EBADFD;
-	snd_timer_stop(dshare->timer);
+	snd_pcm_direct_timer_stop(dshare);
 	do_silence(pcm);
 	dshare->state = SND_PCM_STATE_SETUP;
 	return 0;
@@ -323,51 +371,57 @@ static int snd_pcm_dshare_drain(snd_pcm_t *pcm)
 
 	if (dshare->state == SND_PCM_STATE_OPEN)
 		return -EBADFD;
+	if (pcm->mode & SND_PCM_NONBLOCK)
+		return -EAGAIN;
+	if (dshare->state == SND_PCM_STATE_PREPARED) {
+		if (snd_pcm_mmap_playback_hw_avail(pcm) > 0)
+			snd_pcm_dshare_start(pcm);
+		else {
+			snd_pcm_dshare_drop(pcm);
+			return 0;
+		}
+	}
 	stop_threshold = pcm->stop_threshold;
 	if (pcm->stop_threshold > pcm->buffer_size)
 		pcm->stop_threshold = pcm->buffer_size;
-	while (dshare->state == SND_PCM_STATE_RUNNING) {
+	dshare->state = SND_PCM_STATE_DRAINING;
+	do {
 		err = snd_pcm_dshare_sync_ptr(pcm);
-		if (err < 0)
+		if (err < 0) {
+			snd_pcm_dshare_drop(pcm);
 			break;
-		if (pcm->mode & SND_PCM_NONBLOCK)
-			return -EAGAIN;
-		snd_pcm_wait(pcm, -1);
-	}
+		}
+		if (dshare->state == SND_PCM_STATE_DRAINING) {
+			snd_pcm_dshare_sync_area(pcm);
+			snd_pcm_wait_nocheck(pcm, -1);
+			snd_pcm_direct_clear_timer_queue(dshare); /* force poll to wait */
+		}
+	} while (dshare->state == SND_PCM_STATE_DRAINING);
 	pcm->stop_threshold = stop_threshold;
-	return snd_pcm_dshare_drop(pcm);
-}
-
-static int snd_pcm_dshare_pause(snd_pcm_t *pcm, int enable)
-{
-	snd_pcm_direct_t *dshare = pcm->private_data;
-        if (enable) {
-		if (dshare->state != SND_PCM_STATE_RUNNING)
-			return -EBADFD;
-		dshare->state = SND_PCM_STATE_PAUSED;
-		snd_timer_stop(dshare->timer);
-		do_silence(pcm);
-	} else {
-		if (dshare->state != SND_PCM_STATE_PAUSED)
-			return -EBADFD;
-                dshare->state = SND_PCM_STATE_RUNNING;
-                snd_timer_start(dshare->timer);
-	}
 	return 0;
 }
 
-static snd_pcm_sframes_t snd_pcm_dshare_rewind(snd_pcm_t *pcm, snd_pcm_uframes_t frames)
+static int snd_pcm_dshare_pause(snd_pcm_t *pcm ATTRIBUTE_UNUSED, int enable ATTRIBUTE_UNUSED)
 {
+	return -EIO;
+}
+
+static snd_pcm_sframes_t snd_pcm_dshare_rewind(snd_pcm_t *pcm ATTRIBUTE_UNUSED, snd_pcm_uframes_t frames ATTRIBUTE_UNUSED)
+{
+#if 0
 	/* FIXME: substract samples from the mix ring buffer, too? */
 	snd_pcm_mmap_appl_backward(pcm, frames);
 	return frames;
+#else
+	return -EIO;
+#endif
 }
 
 static snd_pcm_sframes_t snd_pcm_dshare_forward(snd_pcm_t *pcm, snd_pcm_uframes_t frames)
 {
 	snd_pcm_sframes_t avail;
 
-	avail = snd_pcm_mmap_avail(pcm);
+	avail = snd_pcm_mmap_playback_avail(pcm);
 	if (avail < 0)
 		return 0;
 	if (frames > (snd_pcm_uframes_t)avail)
@@ -376,10 +430,10 @@ static snd_pcm_sframes_t snd_pcm_dshare_forward(snd_pcm_t *pcm, snd_pcm_uframes_
 	return frames;
 }
 
-static int snd_pcm_dshare_resume(snd_pcm_t *pcm ATTRIBUTE_UNUSED)
+static int snd_pcm_dshare_resume(snd_pcm_t *pcm)
 {
-	// snd_pcm_direct_t *dshare = pcm->private_data;
-	// FIXME
+	snd_pcm_direct_t *dshare = pcm->private_data;
+	snd_pcm_resume(dshare->spcm);
 	return 0;
 }
 
@@ -407,12 +461,8 @@ static int snd_pcm_dshare_close(snd_pcm_t *pcm)
  		snd_pcm_direct_server_discard(dshare);
  	if (dshare->client)
  		snd_pcm_direct_client_discard(dshare);
- 	if (snd_pcm_direct_shm_discard(dshare) > 0) {
- 		if (snd_pcm_direct_semaphore_discard(dshare) < 0)
- 			snd_pcm_direct_semaphore_up(dshare, DIRECT_IPC_SEM_CLIENT);
- 	} else {
-		snd_pcm_direct_semaphore_up(dshare, DIRECT_IPC_SEM_CLIENT);
-	}
+	snd_pcm_direct_shm_discard(dshare);
+	snd_pcm_direct_semaphore_up(dshare, DIRECT_IPC_SEM_CLIENT);
 	if (dshare->bindings)
 		free(dshare->bindings);
 	pcm->private_data = NULL;
@@ -427,28 +477,42 @@ static snd_pcm_sframes_t snd_pcm_dshare_mmap_commit(snd_pcm_t *pcm,
 	snd_pcm_direct_t *dshare = pcm->private_data;
 	int err;
 
+	switch (snd_pcm_state(dshare->spcm)) {
+	case SND_PCM_STATE_XRUN:
+		return -EPIPE;
+	case SND_PCM_STATE_SUSPENDED:
+		return -ESTRPIPE;
+	default:
+		break;
+	}
+	if (! size)
+		return 0;
 	snd_pcm_mmap_appl_forward(pcm, size);
-	if (dshare->state == SND_PCM_STATE_RUNNING) {
-		err = snd_pcm_dshare_sync_ptr(pcm);
-		if (err < 0)
+	if (dshare->state == STATE_RUN_PENDING) {
+		if ((err = snd_pcm_dshare_start_timer(dshare)) < 0)
 			return err;
+	} else if (dshare->state == SND_PCM_STATE_RUNNING ||
+		   dshare->state == SND_PCM_STATE_DRAINING)
+		snd_pcm_dshare_sync_ptr(pcm);
+	if (dshare->state == SND_PCM_STATE_RUNNING ||
+	    dshare->state == SND_PCM_STATE_DRAINING) {
 		/* ok, we commit the changes after the validation of area */
 		/* it's intended, although the result might be crappy */
-		snd_pcm_dshare_sync_area(pcm, size);
+		snd_pcm_dshare_sync_area(pcm);
+		/* clear timer queue to avoid a bogus return from poll */
+		if (snd_pcm_mmap_playback_avail(pcm) < pcm->avail_min)
+			snd_pcm_direct_clear_timer_queue(dshare);
 	}
 	return size;
 }
 
-static snd_pcm_sframes_t snd_pcm_dshare_avail_update(snd_pcm_t *pcm ATTRIBUTE_UNUSED)
+static snd_pcm_sframes_t snd_pcm_dshare_avail_update(snd_pcm_t *pcm)
 {
 	snd_pcm_direct_t *dshare = pcm->private_data;
-	int err;
 	
-	if (dshare->state == SND_PCM_STATE_RUNNING) {
-		err = snd_pcm_dshare_sync_ptr(pcm);
-		if (err < 0)
-			return err;
-	}
+	if (dshare->state == SND_PCM_STATE_RUNNING ||
+	    dshare->state == SND_PCM_STATE_DRAINING)
+		snd_pcm_dshare_sync_ptr(pcm);
 	return snd_pcm_mmap_playback_avail(pcm);
 }
 
@@ -476,7 +540,6 @@ static snd_pcm_ops_t snd_pcm_dshare_ops = {
 	.dump = snd_pcm_dshare_dump,
 	.nonblock = snd_pcm_direct_nonblock,
 	.async = snd_pcm_direct_async,
-	.poll_revents = snd_pcm_direct_poll_revents,
 	.mmap = snd_pcm_direct_mmap,
 	.munmap = snd_pcm_direct_munmap,
 };
@@ -495,13 +558,18 @@ static snd_pcm_fast_ops_t snd_pcm_dshare_fast_ops = {
 	.rewind = snd_pcm_dshare_rewind,
 	.forward = snd_pcm_dshare_forward,
 	.resume = snd_pcm_dshare_resume,
-	.poll_ask = NULL,
+	.link_fd = NULL,
+	.link = NULL,
+	.unlink = NULL,
 	.writei = snd_pcm_mmap_writei,
 	.writen = snd_pcm_mmap_writen,
 	.readi = snd_pcm_dshare_readi,
 	.readn = snd_pcm_dshare_readn,
 	.avail_update = snd_pcm_dshare_avail_update,
 	.mmap_commit = snd_pcm_dshare_mmap_commit,
+	.poll_descriptors = NULL,
+	.poll_descriptors_count = NULL,
+	.poll_revents = snd_pcm_direct_poll_revents,
 };
 
 /**
@@ -509,7 +577,7 @@ static snd_pcm_fast_ops_t snd_pcm_dshare_fast_ops = {
  * \param pcmp Returns created PCM handle
  * \param name Name of PCM
  * \param ipc_key IPC key for semaphore and shared memory
- * \param ipc_mode IPC permissions for semaphore and shared memory
+ * \param ipc_perm IPC permissions for semaphore and shared memory
  * \param params Parameters for slave
  * \param bindings Channel bindings
  * \param slowptr Slow but more precise pointer updates
@@ -546,17 +614,17 @@ int snd_pcm_dshare_open(snd_pcm_t **pcmp, const char *name,
 	dshare = calloc(1, sizeof(snd_pcm_direct_t));
 	if (!dshare) {
 		ret = -ENOMEM;
-		goto _err;
+		goto _err_nosem;
 	}
 	
 	ret = snd_pcm_direct_parse_bindings(dshare, bindings);
 	if (ret < 0)
-		goto _err;
+		goto _err_nosem;
 		
 	if (!dshare->bindings) {
 		SNDERR("dshare: specify bindings!!!");
 		ret = -EINVAL;
-		goto _err;
+		goto _err_nosem;
 	}
 	
 	dshare->ipc_key = ipc_key;
@@ -566,20 +634,20 @@ int snd_pcm_dshare_open(snd_pcm_t **pcmp, const char *name,
 
 	ret = snd_pcm_new(&pcm, dshare->type = SND_PCM_TYPE_DSHARE, name, stream, mode);
 	if (ret < 0)
-		goto _err;
+		goto _err_nosem;
 
 	while (1) {
 		ret = snd_pcm_direct_semaphore_create_or_connect(dshare);
 		if (ret < 0) {
 			SNDERR("unable to create IPC semaphore");
-			goto _err;
+			goto _err_nosem;
 		}
 	
 		ret = snd_pcm_direct_semaphore_down(dshare, DIRECT_IPC_SEM_CLIENT);
 		if (ret < 0) {
 			snd_pcm_direct_semaphore_discard(dshare);
 			if (--fail_sem_loop <= 0)
-				goto _err;
+				goto _err_nosem;
 			continue;
 		}
 		break;
@@ -599,7 +667,7 @@ int snd_pcm_dshare_open(snd_pcm_t **pcmp, const char *name,
 	dshare->sync_ptr = snd_pcm_dshare_sync_ptr;
 
 	if (first_instance) {
-		ret = snd_pcm_open_slave(&spcm, root, sconf, stream, mode);
+		ret = snd_pcm_open_slave(&spcm, root, sconf, stream, mode | SND_PCM_NONBLOCK);
 		if (ret < 0) {
 			SNDERR("unable to open slave");
 			goto _err;
@@ -626,12 +694,15 @@ int snd_pcm_dshare_open(snd_pcm_t **pcmp, const char *name,
 
 		dshare->shmptr->type = spcm->type;
 	} else {
+		/* up semaphore to avoid deadlock */
+		snd_pcm_direct_semaphore_up(dshare, DIRECT_IPC_SEM_CLIENT);
 		ret = snd_pcm_direct_client_connect(dshare);
 		if (ret < 0) {
 			SNDERR("unable to connect client");
-			return ret;
+			goto _err_nosem;
 		}
 			
+		snd_pcm_direct_semaphore_down(dshare, DIRECT_IPC_SEM_CLIENT);
 		ret = snd_pcm_hw_open_fd(&spcm, "dshare_client", dshare->hw_fd, 0, 0);
 		if (ret < 0) {
 			SNDERR("unable to open hardware");
@@ -683,27 +754,24 @@ int snd_pcm_dshare_open(snd_pcm_t **pcmp, const char *name,
 	return 0;
 	
  _err:
+	if (dshare->shmptr)
+		dshare->shmptr->u.dshare.chn_mask &= ~dshare->u.dshare.chn_mask;
+	if (dshare->timer)
+		snd_timer_close(dshare->timer);
+	if (dshare->server)
+		snd_pcm_direct_server_discard(dshare);
+	if (dshare->client)
+		snd_pcm_direct_client_discard(dshare);
+	if (spcm)
+		snd_pcm_close(spcm);
+	if (dshare->shmid >= 0)
+		snd_pcm_direct_shm_discard(dshare);
+	if (snd_pcm_direct_semaphore_discard(dshare) < 0)
+		snd_pcm_direct_semaphore_up(dshare, DIRECT_IPC_SEM_CLIENT);
+ _err_nosem:
 	if (dshare) {
-		if (dshare->shmptr)
-			dshare->shmptr->u.dshare.chn_mask &= ~dshare->u.dshare.chn_mask;
-	 	if (dshare->timer)
- 			snd_timer_close(dshare->timer);
- 		if (dshare->server)
- 			snd_pcm_direct_server_discard(dshare);
- 		if (dshare->client)
- 			snd_pcm_direct_client_discard(dshare);
- 		if (spcm)
- 			snd_pcm_close(spcm);
- 		if (dshare->shmid >= 0) {
- 			if (snd_pcm_direct_shm_discard(dshare) > 0) {
-			 	if (dshare->semid >= 0) {
- 					if (snd_pcm_direct_semaphore_discard(dshare) < 0)
- 						snd_pcm_direct_semaphore_up(dshare, DIRECT_IPC_SEM_CLIENT);
- 				}
- 			}
- 		}
- 		if (dshare->bindings)
- 			free(dshare->bindings);
+		if (dshare->bindings)
+			free(dshare->bindings);
 		free(dshare);
 	}
 	if (pcm)
@@ -818,14 +886,7 @@ int _snd_pcm_dshare_open(snd_pcm_t **pcmp, const char *name,
 			continue;
 		}
 		if (strcmp(id, "ipc_key_add_uid") == 0) {
-			char *tmp;
-			err = snd_config_get_ascii(n, &tmp);
-			if (err < 0) {
-				SNDERR("The field ipc_key_add_uid must be a boolean type");
-				return err;
-			}
-			err = snd_config_get_bool_ascii(tmp);
-			free(tmp);
+			err = snd_config_get_bool(n);
 			if (err < 0) {
 				SNDERR("The field ipc_key_add_uid must be a boolean type");
 				return err;
@@ -842,14 +903,7 @@ int _snd_pcm_dshare_open(snd_pcm_t **pcmp, const char *name,
 			continue;
 		}
 		if (strcmp(id, "slowptr") == 0) {
-			char *tmp;
-			err = snd_config_get_ascii(n, &tmp);
-			if (err < 0) {
-				SNDERR("The field slowptr must be a boolean type");
-				return err;
-			}
-			err = snd_config_get_bool_ascii(tmp);
-			free(tmp);
+			err = snd_config_get_bool(n);
 			if (err < 0) {
 				SNDERR("The field slowptr must be a boolean type");
 				return err;
@@ -874,7 +928,7 @@ int _snd_pcm_dshare_open(snd_pcm_t **pcmp, const char *name,
 	params.format = SND_PCM_FORMAT_S16;
 	params.rate = 48000;
 	params.channels = 2;
-	params.period_time = 125000;	/* 0.125 seconds */
+	params.period_time = -1;
 	params.buffer_time = -1;
 	bsize = psize = -1;
 	params.periods = 3;
@@ -890,6 +944,10 @@ int _snd_pcm_dshare_open(snd_pcm_t **pcmp, const char *name,
 	if (err < 0)
 		return err;
 
+	/* set a reasonable default */
+	if (psize == -1 && params.period_time == -1)
+		params.period_time = 125000;	/* 0.125 seconds */
+
 	params.period_size = psize;
 	params.buffer_size = bsize;
 	err = snd_pcm_dshare_open(pcmp, name, ipc_key, ipc_perm, &params, bindings, slowptr, root, sconf, stream, mode);
