@@ -138,6 +138,7 @@ static inline int32_t iec958_to_s32(snd_pcm_iec958_t *iec, u_int32_t data)
 	return (int32_t)data;
 }
 
+#ifndef DOC_HIDDEN
 static void snd_pcm_iec958_decode(snd_pcm_iec958_t *iec,
 				  const snd_pcm_channel_area_t *dst_areas,
 				  snd_pcm_uframes_t dst_offset,
@@ -218,6 +219,7 @@ static void snd_pcm_iec958_encode(snd_pcm_iec958_t *iec,
 		}
 	}
 }
+#endif /* DOC_HIDDEN */
 
 static int snd_pcm_iec958_hw_refine_cprepare(snd_pcm_t *pcm, snd_pcm_hw_params_t *params)
 {
@@ -305,7 +307,7 @@ static int snd_pcm_iec958_hw_refine(snd_pcm_t *pcm, snd_pcm_hw_params_t *params)
 				       snd_pcm_iec958_hw_refine_cchange,
 				       snd_pcm_iec958_hw_refine_sprepare,
 				       snd_pcm_iec958_hw_refine_schange,
-				       snd_pcm_plugin_hw_refine_slave);
+				       snd_pcm_generic_hw_refine);
 }
 
 static int snd_pcm_iec958_hw_params(snd_pcm_t *pcm, snd_pcm_hw_params_t * params)
@@ -316,7 +318,7 @@ static int snd_pcm_iec958_hw_params(snd_pcm_t *pcm, snd_pcm_hw_params_t * params
 					  snd_pcm_iec958_hw_refine_cchange,
 					  snd_pcm_iec958_hw_refine_sprepare,
 					  snd_pcm_iec958_hw_refine_schange,
-					  snd_pcm_plugin_hw_params_slave);
+					  snd_pcm_generic_hw_params);
 	if (err < 0)
 		return err;
 
@@ -409,23 +411,22 @@ static void snd_pcm_iec958_dump(snd_pcm_t *pcm, snd_output_t *out)
 		snd_pcm_dump_setup(pcm, out);
 	}
 	snd_output_printf(out, "Slave: ");
-	snd_pcm_dump(iec->plug.slave, out);
+	snd_pcm_dump(iec->plug.gen.slave, out);
 }
 
 static snd_pcm_ops_t snd_pcm_iec958_ops = {
-	.close = snd_pcm_plugin_close,
-	.info = snd_pcm_plugin_info,
+	.close = snd_pcm_generic_close,
+	.info = snd_pcm_generic_info,
 	.hw_refine = snd_pcm_iec958_hw_refine,
 	.hw_params = snd_pcm_iec958_hw_params,
-	.hw_free = snd_pcm_plugin_hw_free,
-	.sw_params = snd_pcm_plugin_sw_params,
-	.channel_info = snd_pcm_plugin_channel_info,
+	.hw_free = snd_pcm_generic_hw_free,
+	.sw_params = snd_pcm_generic_sw_params,
+	.channel_info = snd_pcm_generic_channel_info,
 	.dump = snd_pcm_iec958_dump,
-	.nonblock = snd_pcm_plugin_nonblock,
-	.async = snd_pcm_plugin_async,
-	.poll_revents = snd_pcm_plugin_poll_revents,
-	.mmap = snd_pcm_plugin_mmap,
-	.munmap = snd_pcm_plugin_munmap,
+	.nonblock = snd_pcm_generic_nonblock,
+	.async = snd_pcm_generic_async,
+	.mmap = snd_pcm_generic_mmap,
+	.munmap = snd_pcm_generic_munmap,
 };
 
 /**
@@ -435,6 +436,8 @@ static snd_pcm_ops_t snd_pcm_iec958_ops = {
  * \param sformat Slave (destination) format
  * \param slave Slave PCM handle
  * \param close_slave When set, the slave PCM handle is closed with copy PCM
+ * \param status_bits The IEC958 status bits
+ * \param preamble_vals The preamble byte values
  * \retval zero on success otherwise a negative error code
  * \warning Using of this function might be dangerous in the sense
  *          of compatibility reasons. The prototype might be freely
@@ -471,8 +474,8 @@ int snd_pcm_iec958_open(snd_pcm_t **pcmp, const char *name, snd_pcm_format_t sfo
 	iec->plug.init = snd_pcm_iec958_init;
 	iec->plug.undo_read = snd_pcm_plugin_undo_read_generic;
 	iec->plug.undo_write = snd_pcm_plugin_undo_write_generic;
-	iec->plug.slave = slave;
-	iec->plug.close_slave = close_slave;
+	iec->plug.gen.slave = slave;
+	iec->plug.gen.close_slave = close_slave;
 
 	if (status_bits)
 		memcpy(iec->status, status_bits, sizeof(iec->status));
