@@ -304,7 +304,7 @@ static int snd_pcm_linear_hw_refine(snd_pcm_t *pcm, snd_pcm_hw_params_t *params)
 				       snd_pcm_linear_hw_refine_cchange,
 				       snd_pcm_linear_hw_refine_sprepare,
 				       snd_pcm_linear_hw_refine_schange,
-				       snd_pcm_plugin_hw_refine_slave);
+				       snd_pcm_generic_hw_refine);
 }
 
 static int snd_pcm_linear_hw_params(snd_pcm_t *pcm, snd_pcm_hw_params_t *params)
@@ -315,7 +315,7 @@ static int snd_pcm_linear_hw_params(snd_pcm_t *pcm, snd_pcm_hw_params_t *params)
 					  snd_pcm_linear_hw_refine_cchange,
 					  snd_pcm_linear_hw_refine_sprepare,
 					  snd_pcm_linear_hw_refine_schange,
-					  snd_pcm_plugin_hw_params_slave);
+					  snd_pcm_generic_hw_params);
 	if (err < 0)
 		return err;
 	err = INTERNAL(snd_pcm_hw_params_get_format)(params, &format);
@@ -402,23 +402,22 @@ static void snd_pcm_linear_dump(snd_pcm_t *pcm, snd_output_t *out)
 		snd_pcm_dump_setup(pcm, out);
 	}
 	snd_output_printf(out, "Slave: ");
-	snd_pcm_dump(linear->plug.slave, out);
+	snd_pcm_dump(linear->plug.gen.slave, out);
 }
 
 static snd_pcm_ops_t snd_pcm_linear_ops = {
-	.close = snd_pcm_plugin_close,
-	.info = snd_pcm_plugin_info,
+	.close = snd_pcm_generic_close,
+	.info = snd_pcm_generic_info,
 	.hw_refine = snd_pcm_linear_hw_refine,
 	.hw_params = snd_pcm_linear_hw_params,
-	.hw_free = snd_pcm_plugin_hw_free,
-	.sw_params = snd_pcm_plugin_sw_params,
-	.channel_info = snd_pcm_plugin_channel_info,
+	.hw_free = snd_pcm_generic_hw_free,
+	.sw_params = snd_pcm_generic_sw_params,
+	.channel_info = snd_pcm_generic_channel_info,
 	.dump = snd_pcm_linear_dump,
-	.nonblock = snd_pcm_plugin_nonblock,
-	.async = snd_pcm_plugin_async,
-	.poll_revents = snd_pcm_plugin_poll_revents,
-	.mmap = snd_pcm_plugin_mmap,
-	.munmap = snd_pcm_plugin_munmap,
+	.nonblock = snd_pcm_generic_nonblock,
+	.async = snd_pcm_generic_async,
+	.mmap = snd_pcm_generic_mmap,
+	.munmap = snd_pcm_generic_munmap,
 };
 
 
@@ -452,8 +451,8 @@ int snd_pcm_linear_open(snd_pcm_t **pcmp, const char *name, snd_pcm_format_t sfo
 	linear->plug.write = snd_pcm_linear_write_areas;
 	linear->plug.undo_read = snd_pcm_plugin_undo_read_generic;
 	linear->plug.undo_write = snd_pcm_plugin_undo_write_generic;
-	linear->plug.slave = slave;
-	linear->plug.close_slave = close_slave;
+	linear->plug.gen.slave = slave;
+	linear->plug.gen.close_slave = close_slave;
 
 	err = snd_pcm_new(&pcm, SND_PCM_TYPE_LINEAR, name, slave->stream, slave->mode);
 	if (err < 0) {
@@ -488,6 +487,7 @@ pcm.name {
                 pcm STR         # Slave PCM name
                 # or
                 pcm { }         # Slave PCM definition
+                format STR      # Slave format
         }
 }
 \endcode
