@@ -45,9 +45,8 @@ enum snd_pcm_plug_route_policy {
 };
 
 typedef struct {
+	snd_pcm_generic_t gen;
 	snd_pcm_t *req_slave;
-	int close_slave;
-	snd_pcm_t *slave;
 	snd_pcm_format_t sformat;
 	int schannels;
 	int srate;
@@ -65,11 +64,11 @@ static int snd_pcm_plug_close(snd_pcm_t *pcm)
 	int err, result = 0;
 	if (plug->ttable)
 		free(plug->ttable);
-	if (plug->slave != plug->req_slave) {
+	if (plug->gen.slave != plug->req_slave) {
 		SNDERR("plug slaves mismatch");
 		return -EINVAL;
 	}
-	if (plug->close_slave) {
+	if (plug->gen.close_slave) {
 		snd_pcm_unlink_hw_ptr(pcm, plug->req_slave);
 		snd_pcm_unlink_appl_ptr(pcm, plug->req_slave);
 		err = snd_pcm_close(plug->req_slave);
@@ -80,24 +79,6 @@ static int snd_pcm_plug_close(snd_pcm_t *pcm)
 	return result;
 }
 
-static int snd_pcm_plug_nonblock(snd_pcm_t *pcm, int nonblock)
-{
-	snd_pcm_plug_t *plug = pcm->private_data;
-	return snd_pcm_nonblock(plug->slave, nonblock);
-}
-
-static int snd_pcm_plug_async(snd_pcm_t *pcm, int sig, pid_t pid)
-{
-	snd_pcm_plug_t *plug = pcm->private_data;
-	return snd_pcm_async(plug->slave, sig, pid);
-}
-
-static int snd_pcm_plug_poll_revents(snd_pcm_t *pcm, struct pollfd *pfds, unsigned int nfds, unsigned short *revents)
-{
-	snd_pcm_plug_t *plug = pcm->private_data;
-	return snd_pcm_poll_descriptors_revents(plug->slave, pfds, nfds, revents);
-}
-
 static int snd_pcm_plug_info(snd_pcm_t *pcm, snd_pcm_info_t *info)
 {
 	snd_pcm_plug_t *plug = pcm->private_data;
@@ -320,22 +301,24 @@ static void snd_pcm_plug_clear(snd_pcm_t *pcm)
 	snd_pcm_plug_t *plug = pcm->private_data;
 	snd_pcm_t *slave = plug->req_slave;
 	/* Clear old plugins */
-	if (plug->slave != slave) {
-		snd_pcm_unlink_hw_ptr(pcm, plug->slave);
-		snd_pcm_unlink_appl_ptr(pcm, plug->slave);
-		snd_pcm_close(plug->slave);
-		plug->slave = slave;
+	if (plug->gen.slave != slave) {
+		snd_pcm_unlink_hw_ptr(pcm, plug->gen.slave);
+		snd_pcm_unlink_appl_ptr(pcm, plug->gen.slave);
+		snd_pcm_close(plug->gen.slave);
+		plug->gen.slave = slave;
 		pcm->fast_ops = slave->fast_ops;
 		pcm->fast_op_arg = slave->fast_op_arg;
 	}
 }
 
+#ifndef DOC_HIDDEN
 typedef struct {
 	snd_pcm_access_t access;
 	snd_pcm_format_t format;
 	unsigned int channels;
 	unsigned int rate;
 } snd_pcm_plug_params_t;
+#endif
 
 static int snd_pcm_plug_change_rate(snd_pcm_t *pcm, snd_pcm_t **new, snd_pcm_plug_params_t *clt, snd_pcm_plug_params_t *slv)
 {
@@ -344,7 +327,7 @@ static int snd_pcm_plug_change_rate(snd_pcm_t *pcm, snd_pcm_t **new, snd_pcm_plu
 	assert(snd_pcm_format_linear(slv->format));
 	if (clt->rate == slv->rate)
 		return 0;
-	err = snd_pcm_rate_open(new, NULL, slv->format, slv->rate, plug->slave, plug->slave != plug->req_slave);
+	err = snd_pcm_rate_open(new, NULL, slv->format, slv->rate, plug->gen.slave, plug->gen.slave != plug->req_slave);
 	if (err < 0)
 		return err;
 	slv->access = clt->access;
@@ -445,7 +428,7 @@ static int snd_pcm_plug_change_channels(snd_pcm_t *pcm, snd_pcm_t **new, snd_pcm
 			break;
 		}
 	}
-	err = snd_pcm_route_open(new, NULL, slv->format, (int) slv->channels, ttable, tt_ssize, tt_cused, tt_sused, plug->slave, plug->slave != plug->req_slave);
+	err = snd_pcm_route_open(new, NULL, slv->format, (int) slv->channels, ttable, tt_ssize, tt_cused, tt_sused, plug->gen.slave, plug->gen.slave != plug->req_slave);
 	if (err < 0)
 		return err;
 	slv->channels = clt->channels;
@@ -525,7 +508,7 @@ static int snd_pcm_plug_change_format(snd_pcm_t *pcm, snd_pcm_t **new, snd_pcm_p
 		else
 			cfmt = SND_PCM_FORMAT_S16;
 	}
-	err = f(new, NULL, slv->format, plug->slave, plug->slave != plug->req_slave);
+	err = f(new, NULL, slv->format, plug->gen.slave, plug->gen.slave != plug->req_slave);
 	if (err < 0)
 		return err;
 	slv->format = cfmt;
@@ -539,7 +522,7 @@ static int snd_pcm_plug_change_access(snd_pcm_t *pcm, snd_pcm_t **new, snd_pcm_p
 	int err;
 	if (clt->access == slv->access)
 		return 0;
-	err = snd_pcm_copy_open(new, NULL, plug->slave, plug->slave != plug->req_slave);
+	err = snd_pcm_copy_open(new, NULL, plug->gen.slave, plug->gen.slave != plug->req_slave);
 	if (err < 0)
 		return err;
 	slv->access = clt->access;
@@ -551,7 +534,7 @@ static int snd_pcm_plug_insert_plugins(snd_pcm_t *pcm,
 				       snd_pcm_plug_params_t *slave)
 {
 	snd_pcm_plug_t *plug = pcm->private_data;
-	int (*funcs[])(snd_pcm_t *_pcm, snd_pcm_t **new, snd_pcm_plug_params_t *s, snd_pcm_plug_params_t *d) = {
+	static int (*funcs[])(snd_pcm_t *_pcm, snd_pcm_t **new, snd_pcm_plug_params_t *s, snd_pcm_plug_params_t *d) = {
 		snd_pcm_plug_change_format,
 		snd_pcm_plug_change_channels,
 		snd_pcm_plug_change_rate,
@@ -575,7 +558,7 @@ static int snd_pcm_plug_insert_plugins(snd_pcm_t *pcm,
 			return err;
 		}
 		if (err) {
-			plug->slave = new;
+			plug->gen.slave = new;
 			pcm->fast_ops = new->fast_ops;
 			pcm->fast_op_arg = new->fast_op_arg;
 		}
@@ -593,7 +576,7 @@ static int snd_pcm_plug_insert_plugins(snd_pcm_t *pcm,
 		}
 		assert(err);
 		assert(plug->ttable_ok);
-		plug->slave = new;
+		plug->gen.slave = new;
 		pcm->fast_ops = new->fast_ops;
 		pcm->fast_op_arg = new->fast_op_arg;
 	}
@@ -747,6 +730,8 @@ static int snd_pcm_plug_hw_refine_schange(snd_pcm_t *pcm, snd_pcm_hw_params_t *p
 	err = _snd_pcm_hw_params_refine(sparams, links, params);
 	if (err < 0)
 		return err;
+	if (params->flags & SND_PCM_HW_PARAMS_NORESAMPLE)
+		snd_interval_copy((snd_interval_t *)snd_pcm_hw_param_get_interval(params, SND_PCM_HW_PARAM_RATE), snd_pcm_hw_param_get_interval(sparams, SND_PCM_HW_PARAM_RATE));
 	return 0;
 }
 	
@@ -849,6 +834,8 @@ static int snd_pcm_plug_hw_refine_cchange(snd_pcm_t *pcm ATTRIBUTE_UNUSED,
 	err = _snd_pcm_hw_params_refine(params, links, sparams);
 	if (err < 0)
 		return err;
+	if (params->flags & SND_PCM_HW_PARAMS_NORESAMPLE)
+		snd_interval_copy((snd_interval_t *)snd_pcm_hw_param_get_interval(params, SND_PCM_HW_PARAM_RATE), snd_pcm_hw_param_get_interval(sparams, SND_PCM_HW_PARAM_RATE));
 	/* FIXME */
 	params->info &= ~(SND_PCM_INFO_MMAP | SND_PCM_INFO_MMAP_VALID);
 	return 0;
@@ -908,7 +895,7 @@ static int snd_pcm_plug_hw_params(snd_pcm_t *pcm, snd_pcm_hw_params_t *params)
 		if (err < 0)
 			return err;
 	}
-	slave = plug->slave;
+	slave = plug->gen.slave;
 	err = _snd_pcm_hw_params(slave, params);
 	if (err < 0) {
 		snd_pcm_plug_clear(pcm);
@@ -924,30 +911,18 @@ static int snd_pcm_plug_hw_params(snd_pcm_t *pcm, snd_pcm_hw_params_t *params)
 static int snd_pcm_plug_hw_free(snd_pcm_t *pcm)
 {
 	snd_pcm_plug_t *plug = pcm->private_data;
-	snd_pcm_t *slave = plug->slave;
+	snd_pcm_t *slave = plug->gen.slave;
 	int err = snd_pcm_hw_free(slave);
 	snd_pcm_plug_clear(pcm);
 	return err;
 }
 
-static int snd_pcm_plug_sw_params(snd_pcm_t *pcm, snd_pcm_sw_params_t * params)
-{
-	snd_pcm_plug_t *plug = pcm->private_data;
-	return snd_pcm_sw_params(plug->slave, params);
-}
-
-static int snd_pcm_plug_channel_info(snd_pcm_t *pcm, snd_pcm_channel_info_t *info)
-{
-	snd_pcm_plug_t *plug = pcm->private_data;
-	return snd_pcm_channel_info(plug->slave, info);
-}
-
 static int snd_pcm_plug_mmap(snd_pcm_t *pcm)
 {
 	snd_pcm_plug_t *plug = pcm->private_data;
-	pcm->mmap_channels = plug->slave->mmap_channels;
-	pcm->running_areas = plug->slave->running_areas;
-	pcm->stopped_areas = plug->slave->stopped_areas;
+	pcm->mmap_channels = plug->gen.slave->mmap_channels;
+	pcm->running_areas = plug->gen.slave->running_areas;
+	pcm->stopped_areas = plug->gen.slave->stopped_areas;
 	pcm->mmap_shadow = 1;
 	return 0;
 }
@@ -966,7 +941,7 @@ static void snd_pcm_plug_dump(snd_pcm_t *pcm, snd_output_t *out)
 {
 	snd_pcm_plug_t *plug = pcm->private_data;
 	snd_output_printf(out, "Plug PCM: ");
-	snd_pcm_dump(plug->slave, out);
+	snd_pcm_dump(plug->gen.slave, out);
 }
 
 static snd_pcm_ops_t snd_pcm_plug_ops = {
@@ -975,12 +950,11 @@ static snd_pcm_ops_t snd_pcm_plug_ops = {
 	.hw_refine = snd_pcm_plug_hw_refine,
 	.hw_params = snd_pcm_plug_hw_params,
 	.hw_free = snd_pcm_plug_hw_free,
-	.sw_params = snd_pcm_plug_sw_params,
-	.channel_info = snd_pcm_plug_channel_info,
+	.sw_params = snd_pcm_generic_sw_params,
+	.channel_info = snd_pcm_generic_channel_info,
 	.dump = snd_pcm_plug_dump,
-	.nonblock = snd_pcm_plug_nonblock,
-	.async = snd_pcm_plug_async,
-	.poll_revents = snd_pcm_plug_poll_revents,
+	.nonblock = snd_pcm_generic_nonblock,
+	.async = snd_pcm_generic_async,
 	.mmap = snd_pcm_plug_mmap,
 	.munmap = snd_pcm_plug_munmap,
 };
@@ -1017,8 +991,8 @@ int snd_pcm_plug_open(snd_pcm_t **pcmp,
 	plug->sformat = sformat;
 	plug->schannels = schannels;
 	plug->srate = srate;
-	plug->slave = plug->req_slave = slave;
-	plug->close_slave = close_slave;
+	plug->gen.slave = plug->req_slave = slave;
+	plug->gen.close_slave = close_slave;
 	plug->route_policy = route_policy;
 	plug->ttable = ttable;
 	plug->tt_ssize = tt_ssize;
