@@ -66,10 +66,10 @@ typedef void (*rate_f)(const snd_pcm_channel_area_t *dst_areas,
 		       snd_pcm_rate_t *rate);
 
 struct _snd_pcm_rate {
-	snd_pcm_t *slave;
-	int close_slave;
+	snd_pcm_generic_t gen;
 	snd_atomic_write_t watom;
 	snd_pcm_uframes_t appl_ptr, hw_ptr;
+	snd_pcm_uframes_t last_commit_ptr;
 	snd_pcm_uframes_t orig_avail_min;
 	snd_pcm_sw_params_t sw_params;
 	enum rate_type type;
@@ -82,6 +82,7 @@ struct _snd_pcm_rate {
 	unsigned int srate;
 	snd_pcm_channel_area_t *pareas;	/* areas for splitted period (rate pcm) */
 	snd_pcm_channel_area_t *sareas;	/* areas for splitted period (slave pcm) */
+	int16_t *old_sample;
 };
 
 static void snd_pcm_rate_expand(const snd_pcm_channel_area_t *dst_areas,
@@ -103,6 +104,7 @@ static void snd_pcm_rate_expand(const snd_pcm_channel_area_t *dst_areas,
 	snd_pcm_uframes_t src_frames1;
 	snd_pcm_uframes_t dst_frames1;
 	int16_t sample = 0;
+	unsigned int pos;
 	
 	for (channel = 0; channel < channels; ++channel) {
 		const snd_pcm_channel_area_t *src_area = &src_areas[channel];
@@ -111,35 +113,28 @@ static void snd_pcm_rate_expand(const snd_pcm_channel_area_t *dst_areas,
 		char *dst;
 		int src_step, dst_step;
 		int16_t old_sample = 0;
-		int16_t new_sample = 0;
+		int16_t new_sample;
 		int old_weight, new_weight;
-		unsigned int pos = 0;
-		int init;
 		src = snd_pcm_channel_area_addr(src_area, src_offset);
 		dst = snd_pcm_channel_area_addr(dst_area, dst_offset);
 		src_step = snd_pcm_channel_area_step(src_area);
 		dst_step = snd_pcm_channel_area_step(dst_area);
 		src_frames1 = 0;
 		dst_frames1 = 0;
-		init = 1;
+		new_sample = rate->old_sample[channel];
+		pos = get_threshold;
 		while (dst_frames1 < dst_frames) {
 			if (pos >= get_threshold) {
-				src += src_step;
-				src_frames1++;
+				pos -= get_threshold;
+				old_sample = new_sample;
 				if (src_frames1 < src_frames) {
-					old_sample = new_sample;
 					goto *get;
 #define GET16_END after_get
 #include "plugin_ops.h"
 #undef GET16_END
 				after_get:
 					new_sample = sample;
-					if (init) {
-						init = 0;
-						continue;
-					}
 				}
-				pos -= get_threshold;
 			}
 			new_weight = (pos << (16 - rate->pitch_shift)) / (get_threshold >> rate->pitch_shift);
 			old_weight = 0x10000 - new_weight;
@@ -152,7 +147,12 @@ static void snd_pcm_rate_expand(const snd_pcm_channel_area_t *dst_areas,
 			dst += dst_step;
 			dst_frames1++;
 			pos += LINEAR_DIV;
+			if (pos >= get_threshold) {
+				src += src_step;
+				src_frames1++;
+			}
 		} 
+		rate->old_sample[channel] = new_sample;
 	}
 }
 
@@ -168,6 +168,7 @@ static void snd_pcm_rate_expand_s16(const snd_pcm_channel_area_t *dst_areas,
 	snd_pcm_uframes_t src_frames1;
 	snd_pcm_uframes_t dst_frames1;
 	unsigned int get_threshold = rate->pitch;
+	unsigned int pos;
 	
 	for (channel = 0; channel < channels; ++channel) {
 		const snd_pcm_channel_area_t *src_area = &src_areas[channel];
@@ -175,27 +176,23 @@ static void snd_pcm_rate_expand_s16(const snd_pcm_channel_area_t *dst_areas,
 		const int16_t *src;
 		int16_t *dst;
 		int src_step, dst_step;
-		int16_t old_sample;
+		int16_t old_sample = 0;
 		int16_t new_sample;
 		int old_weight, new_weight;
-		unsigned int pos;
 		src = snd_pcm_channel_area_addr(src_area, src_offset);
 		dst = snd_pcm_channel_area_addr(dst_area, dst_offset);
 		src_step = snd_pcm_channel_area_step(src_area) >> 1;
 		dst_step = snd_pcm_channel_area_step(dst_area) >> 1;
 		src_frames1 = 0;
 		dst_frames1 = 0;
-		old_sample = new_sample = *src;
+		new_sample = rate->old_sample[channel];
 		pos = get_threshold;
 		while (dst_frames1 < dst_frames) {
 			if (pos >= get_threshold) {
 				pos -= get_threshold;
-				src += src_step;
-				src_frames1++;
-				if (src_frames1 < src_frames) {
-					old_sample = new_sample;
+				old_sample = new_sample;
+				if (src_frames1 < src_frames)
 					new_sample = *src;
-				}
 			}
 			new_weight = (pos << (16 - rate->pitch_shift)) / (get_threshold >> rate->pitch_shift);
 			old_weight = 0x10000 - new_weight;
@@ -203,7 +200,12 @@ static void snd_pcm_rate_expand_s16(const snd_pcm_channel_area_t *dst_areas,
 			dst += dst_step;
 			dst_frames1++;
 			pos += LINEAR_DIV;
+			if (pos >= get_threshold) {
+				src += src_step;
+				src_frames1++;
+			}
 		} 
+		rate->old_sample[channel] = new_sample;
 	}
 }
 
@@ -497,18 +499,6 @@ static int snd_pcm_rate_hw_refine_cchange(snd_pcm_t *pcm, snd_pcm_hw_params_t *p
 	return 0;
 }
 
-static int snd_pcm_rate_hw_refine_slave(snd_pcm_t *pcm, snd_pcm_hw_params_t *params)
-{
-	snd_pcm_rate_t *rate = pcm->private_data;
-	return snd_pcm_hw_refine(rate->slave, params);
-}
-
-static int snd_pcm_rate_hw_params_slave(snd_pcm_t *pcm, snd_pcm_hw_params_t *params)
-{
-	snd_pcm_rate_t *rate = pcm->private_data;
-	return _snd_pcm_hw_params(rate->slave, params);
-}
-
 static int snd_pcm_rate_hw_refine(snd_pcm_t *pcm, 
 				  snd_pcm_hw_params_t *params)
 {
@@ -517,13 +507,13 @@ static int snd_pcm_rate_hw_refine(snd_pcm_t *pcm,
 				       snd_pcm_rate_hw_refine_cchange,
 				       snd_pcm_rate_hw_refine_sprepare,
 				       snd_pcm_rate_hw_refine_schange,
-				       snd_pcm_rate_hw_refine_slave);
+				       snd_pcm_generic_hw_refine);
 }
 
 static int snd_pcm_rate_hw_params(snd_pcm_t *pcm, snd_pcm_hw_params_t * params)
 {
 	snd_pcm_rate_t *rate = pcm->private_data;
-	snd_pcm_t *slave = rate->slave;
+	snd_pcm_t *slave = rate->gen.slave;
 	snd_pcm_format_t src_format, dst_format, pformat, sformat;
 	unsigned int src_rate, dst_rate, channels, pwidth, swidth, chn;
 	snd_pcm_uframes_t period_size, buffer_size;
@@ -531,7 +521,7 @@ static int snd_pcm_rate_hw_params(snd_pcm_t *pcm, snd_pcm_hw_params_t * params)
 					  snd_pcm_rate_hw_refine_cchange,
 					  snd_pcm_rate_hw_refine_sprepare,
 					  snd_pcm_rate_hw_refine_schange,
-					  snd_pcm_rate_hw_params_slave);
+					  snd_pcm_generic_hw_params);
 	if (err < 0)
 		return err;
 
@@ -584,12 +574,18 @@ static int snd_pcm_rate_hw_params(snd_pcm_t *pcm, snd_pcm_hw_params_t * params)
 		SNDMSG("rate plugin already in use");
 		return -EBUSY;
 	}
+#if 0
 	if ((buffer_size / period_size) * period_size == buffer_size &&
 	    (slave->buffer_size / slave->period_size) * slave->period_size == slave->buffer_size)
 		return 0;
+#endif
 	rate->pareas = malloc(2 * channels * sizeof(*rate->pareas));
 	if (rate->pareas == NULL)
 		return -ENOMEM;
+	free(rate->old_sample);
+	rate->old_sample = malloc(sizeof(*rate->old_sample) * channels);
+	if (rate->old_sample == NULL)
+		return -ENOMEM;
 	pwidth = snd_pcm_format_physical_width(pformat);
 	swidth = snd_pcm_format_physical_width(sformat);
 	rate->pareas[0].addr = malloc(((pwidth * channels * period_size) / 8) +
@@ -620,13 +616,19 @@ static int snd_pcm_rate_hw_free(snd_pcm_t *pcm)
 		rate->pareas = NULL;
 		rate->sareas = NULL;
 	}
-	return snd_pcm_hw_free(rate->slave);
+	free(rate->old_sample);
+	return snd_pcm_hw_free(rate->gen.slave);
+}
+
+static int snd_pcm_rate_channel_info(snd_pcm_t *pcm, snd_pcm_channel_info_t * info)
+{
+	return snd_pcm_channel_info_shm(pcm, info, -1);
 }
 
 static void recalc(snd_pcm_t *pcm, snd_pcm_uframes_t *val)
 {
 	snd_pcm_rate_t *rate = pcm->private_data;
-	snd_pcm_t *slave = rate->slave;
+	snd_pcm_t *slave = rate->gen.slave;
 	unsigned long div;
 
 	if (*val == pcm->buffer_size) {
@@ -643,7 +645,7 @@ static void recalc(snd_pcm_t *pcm, snd_pcm_uframes_t *val)
 static int snd_pcm_rate_sw_params(snd_pcm_t *pcm, snd_pcm_sw_params_t * params)
 {
 	snd_pcm_rate_t *rate = pcm->private_data;
-	snd_pcm_t *slave = rate->slave;
+	snd_pcm_t *slave = rate->gen.slave;
 	snd_pcm_sw_params_t *sparams;
 	snd_pcm_uframes_t boundary1, boundary2;
 
@@ -756,10 +758,14 @@ static int snd_pcm_rate_init(snd_pcm_t *pcm)
 	snd_pcm_rate_t *rate = pcm->private_data;
 	switch (rate->type) {
 	case RATE_TYPE_LINEAR:
+		/* for expand */
+		if (rate->old_sample)
+			memset(rate->old_sample, 0, sizeof(*rate->old_sample) * pcm->channels);
 		break;
 	default:
 		assert(0);
 	}
+	rate->last_commit_ptr = 0;
 	return 0;
 }
 
@@ -771,7 +777,7 @@ snd_pcm_rate_write_areas1(snd_pcm_t *pcm,
 			 snd_pcm_uframes_t slave_offset)
 {
 	snd_pcm_rate_t *rate = pcm->private_data;
-	rate->func(slave_areas, slave_offset, rate->slave->period_size,
+	rate->func(slave_areas, slave_offset, rate->gen.slave->period_size,
 		   areas, offset, pcm->period_size,
 		   pcm->channels, rate);
 	return 0;
@@ -786,7 +792,7 @@ snd_pcm_rate_read_areas1(snd_pcm_t *pcm,
 {
 	snd_pcm_rate_t *rate = pcm->private_data;
 	rate->func(areas, offset, pcm->period_size,
-		   slave_areas, slave_offset, rate->slave->period_size,
+		   slave_areas, slave_offset, rate->gen.slave->period_size,
 		   pcm->channels, rate);
 	return 0;
 }
@@ -796,7 +802,7 @@ static inline snd_pcm_sframes_t snd_pcm_rate_move_applptr(snd_pcm_t *pcm, snd_pc
 	snd_pcm_rate_t *rate = pcm->private_data;
 	snd_pcm_uframes_t orig_appl_ptr, appl_ptr = rate->appl_ptr, slave_appl_ptr;
 	snd_pcm_sframes_t diff, ndiff;
-	snd_pcm_t *slave = rate->slave;
+	snd_pcm_t *slave = rate->gen.slave;
 
 	orig_appl_ptr = rate->appl_ptr;
 	if (frames > 0)
@@ -804,7 +810,7 @@ static inline snd_pcm_sframes_t snd_pcm_rate_move_applptr(snd_pcm_t *pcm, snd_pc
 	else
 		snd_pcm_mmap_appl_backward(pcm, -frames);
 	slave_appl_ptr =
-		(appl_ptr / pcm->period_size) * rate->slave->period_size;
+		(appl_ptr / pcm->period_size) * rate->gen.slave->period_size;
 	diff = slave_appl_ptr - *slave->appl.ptr;
 	if (diff < -(snd_pcm_sframes_t)(slave->boundary / 2)) {
 		diff = (slave->boundary - *slave->appl.ptr) + slave_appl_ptr;
@@ -814,16 +820,16 @@ static inline snd_pcm_sframes_t snd_pcm_rate_move_applptr(snd_pcm_t *pcm, snd_pc
 	if (diff == 0)
 		return frames;
 	if (diff > 0) {
-		ndiff = snd_pcm_forward(rate->slave, diff);
+		ndiff = snd_pcm_forward(rate->gen.slave, diff);
 	} else {
-		ndiff = snd_pcm_rewind(rate->slave, diff);
+		ndiff = snd_pcm_rewind(rate->gen.slave, diff);
 	}
 	if (ndiff < 0)
 		return diff;
 	slave_appl_ptr = *slave->appl.ptr;
 	rate->appl_ptr =
-		(slave_appl_ptr / rate->slave->period_size) * pcm->period_size +
-		snd_pcm_rate_client_frames(pcm, slave_appl_ptr % rate->slave->period_size) +
+		(slave_appl_ptr / rate->gen.slave->period_size) * pcm->period_size +
+		snd_pcm_rate_client_frames(pcm, slave_appl_ptr % rate->gen.slave->period_size) +
 		orig_appl_ptr % pcm->period_size;
 	diff = orig_appl_ptr - rate->appl_ptr;
 	if (diff < -(snd_pcm_sframes_t)(slave->boundary / 2)) {
@@ -832,71 +838,32 @@ static inline snd_pcm_sframes_t snd_pcm_rate_move_applptr(snd_pcm_t *pcm, snd_pc
 		diff = -((slave->boundary - orig_appl_ptr) + rate->appl_ptr);
 	}
 	if (frames < 0)
-		return -diff;
+		diff = -diff;
+
+	rate->last_commit_ptr = rate->appl_ptr - rate->appl_ptr % pcm->period_size;
+
 	return diff;
 }
 
 static inline void snd_pcm_rate_sync_hwptr(snd_pcm_t *pcm)
 {
 	snd_pcm_rate_t *rate = pcm->private_data;
-	snd_pcm_uframes_t slave_hw_ptr = *rate->slave->hw.ptr;
+	snd_pcm_uframes_t slave_hw_ptr = *rate->gen.slave->hw.ptr;
 
 	if (pcm->stream != SND_PCM_STREAM_PLAYBACK)
 		return;
+	/* FIXME: boundary overlap of slave hw_ptr isn't evaluated here!
+	 *        e.g. if slave rate is small... 
+	 */
 	rate->hw_ptr =
-		(slave_hw_ptr / rate->slave->period_size) * pcm->period_size +
-		snd_pcm_rate_client_frames(pcm, slave_hw_ptr % rate->slave->period_size);
-}
-
-static int snd_pcm_rate_close(snd_pcm_t *pcm)
-{
-	snd_pcm_rate_t *rate = pcm->private_data;
-	int err = 0;
-	if (rate->close_slave)
-		err = snd_pcm_close(rate->slave);
-	free(rate);
-	return 0;
-}
-
-static int snd_pcm_rate_nonblock(snd_pcm_t *pcm, int nonblock)
-{
-	snd_pcm_rate_t *rate = pcm->private_data;
-	return snd_pcm_nonblock(rate->slave, nonblock);
-}
-
-static int snd_pcm_rate_async(snd_pcm_t *pcm, int sig, pid_t pid)
-{
-	snd_pcm_rate_t *rate = pcm->private_data;
-	return snd_pcm_async(rate->slave, sig, pid);
-}
-
-static int snd_pcm_rate_poll_revents(snd_pcm_t *pcm, struct pollfd *pfds, unsigned int nfds, unsigned short *revents)
-{
-	snd_pcm_rate_t *rate = pcm->private_data;
-	return snd_pcm_poll_descriptors_revents(rate->slave, pfds, nfds, revents);
-}
-
-static int snd_pcm_rate_info(snd_pcm_t *pcm, snd_pcm_info_t * info)
-{
-	snd_pcm_rate_t *rate = pcm->private_data;
-	return snd_pcm_info(rate->slave, info);
-}
-
-static int snd_pcm_rate_channel_info(snd_pcm_t *pcm, snd_pcm_channel_info_t *info)
-{
-	return snd_pcm_channel_info_shm(pcm, info, -1);
-}
-
-static snd_pcm_state_t snd_pcm_rate_state(snd_pcm_t *pcm)
-{
-	snd_pcm_rate_t *rate = pcm->private_data;
-	return snd_pcm_state(rate->slave);
+		(slave_hw_ptr / rate->gen.slave->period_size) * pcm->period_size +
+		snd_pcm_rate_client_frames(pcm, slave_hw_ptr % rate->gen.slave->period_size);
 }
 
 static int snd_pcm_rate_hwsync(snd_pcm_t *pcm)
 {
 	snd_pcm_rate_t *rate = pcm->private_data;
-	int err = snd_pcm_hwsync(rate->slave);
+	int err = snd_pcm_hwsync(rate->gen.slave);
 	if (err < 0)
 		return err;
 	snd_atomic_write_begin(&rate->watom);
@@ -921,7 +888,7 @@ static int snd_pcm_rate_prepare(snd_pcm_t *pcm)
 	int err;
 
 	snd_atomic_write_begin(&rate->watom);
-	err = snd_pcm_prepare(rate->slave);
+	err = snd_pcm_prepare(rate->gen.slave);
 	if (err < 0) {
 		snd_atomic_write_end(&rate->watom);
 		return err;
@@ -940,7 +907,7 @@ static int snd_pcm_rate_reset(snd_pcm_t *pcm)
 	snd_pcm_rate_t *rate = pcm->private_data;
 	int err;
 	snd_atomic_write_begin(&rate->watom);
-	err = snd_pcm_reset(rate->slave);
+	err = snd_pcm_reset(rate->gen.slave);
 	if (err < 0) {
 		snd_atomic_write_end(&rate->watom);
 		return err;
@@ -954,30 +921,6 @@ static int snd_pcm_rate_reset(snd_pcm_t *pcm)
 	return 0;
 }
 
-static int snd_pcm_rate_start(snd_pcm_t *pcm)
-{
-	snd_pcm_rate_t *rate = pcm->private_data;
-	return snd_pcm_start(rate->slave);
-}
-
-static int snd_pcm_rate_drop(snd_pcm_t *pcm)
-{
-	snd_pcm_rate_t *rate = pcm->private_data;
-	return snd_pcm_drop(rate->slave);
-}
-
-static int snd_pcm_rate_drain(snd_pcm_t *pcm)
-{
-	snd_pcm_rate_t *rate = pcm->private_data;
-	return snd_pcm_drain(rate->slave);
-}
-
-static int snd_pcm_rate_pause(snd_pcm_t *pcm, int enable)
-{
-	snd_pcm_rate_t *rate = pcm->private_data;
-	return snd_pcm_pause(rate->slave, enable);
-}
-
 static snd_pcm_sframes_t snd_pcm_rate_rewind(snd_pcm_t *pcm, snd_pcm_uframes_t frames)
 {
 	snd_pcm_rate_t *rate = pcm->private_data;
@@ -1010,45 +953,43 @@ static snd_pcm_sframes_t snd_pcm_rate_forward(snd_pcm_t *pcm, snd_pcm_uframes_t
 	return n;
 }
 
-static int snd_pcm_rate_resume(snd_pcm_t *pcm)
-{
-	snd_pcm_rate_t *rate = pcm->private_data;
-	return snd_pcm_resume(rate->slave);
-}
-
-static int snd_pcm_rate_poll_ask(snd_pcm_t *pcm)
+static int snd_pcm_rate_poll_descriptors(snd_pcm_t *pcm, struct pollfd *pfds, unsigned int space)
 {
 	snd_pcm_rate_t *rate = pcm->private_data;
 	snd_pcm_uframes_t avail_min;
-	int err;
+	int ret, err;
+
+	ret = snd_pcm_generic_poll_descriptors(pcm, pfds, space);
+	if (ret < 0)
+		return ret;
 
-	if (rate->slave->fast_ops->poll_ask) {
-		err = rate->slave->fast_ops->poll_ask(rate->slave->fast_op_arg);
-		if (err < 0)
-			return err;
-	}
 	avail_min = rate->appl_ptr % pcm->period_size;
 	if (avail_min > 0) {
 		recalc(pcm, &avail_min);
-		if (avail_min < rate->slave->buffer_size &&
-		    avail_min != rate->slave->period_size)
+		if (avail_min < rate->gen.slave->buffer_size &&
+		    avail_min != rate->gen.slave->period_size)
 			avail_min++;	/* 1st small little rounding correction */
-		if (avail_min < rate->slave->buffer_size &&
-		    avail_min != rate->slave->period_size)
+		if (avail_min < rate->gen.slave->buffer_size &&
+		    avail_min != rate->gen.slave->period_size)
 			avail_min++;	/* 2nd small little rounding correction */
 		avail_min += rate->orig_avail_min;
 	} else {
 		avail_min = rate->orig_avail_min;
 	}
 	if (rate->sw_params.avail_min == avail_min)
-		return 0;
+		return ret;
 	rate->sw_params.avail_min = avail_min;
-	return snd_pcm_sw_params(rate->slave, &rate->sw_params);
+	err = snd_pcm_sw_params(rate->gen.slave, &rate->sw_params);
+	if (err < 0)
+		return err;
+	return ret;
 }
 
-static int snd_pcm_rate_commit_next_period(snd_pcm_t *pcm, snd_pcm_uframes_t appl_offset)
+static int snd_pcm_rate_commit_area(snd_pcm_t *pcm, snd_pcm_rate_t *rate,
+				    snd_pcm_uframes_t appl_offset,
+				    snd_pcm_uframes_t size,
+				    snd_pcm_uframes_t slave_size)
 {
-	snd_pcm_rate_t *rate = pcm->private_data;
 	snd_pcm_uframes_t cont = pcm->buffer_size - appl_offset;
 	const snd_pcm_channel_area_t *areas;
 	const snd_pcm_channel_area_t *slave_areas;
@@ -1057,21 +998,21 @@ static int snd_pcm_rate_commit_next_period(snd_pcm_t *pcm, snd_pcm_uframes_t app
 	snd_pcm_sframes_t result;
 
 	areas = snd_pcm_mmap_areas(pcm);
-	if (cont >= pcm->period_size) {
-		result = snd_pcm_mmap_begin(rate->slave, &slave_areas, &slave_offset, &slave_frames);
+	if (cont >= size) {
+		result = snd_pcm_mmap_begin(rate->gen.slave, &slave_areas, &slave_offset, &slave_frames);
 		if (result < 0)
 			return result;
-		if (slave_frames < rate->slave->period_size) {
+		if (slave_frames < slave_size) {
 			snd_pcm_rate_write_areas1(pcm, areas, appl_offset, rate->sareas, 0);
 			goto __partial;
 		}
 		snd_pcm_rate_write_areas1(pcm, areas, appl_offset,
 					  slave_areas, slave_offset);
-		result = snd_pcm_mmap_commit(rate->slave, slave_offset, rate->slave->period_size);
-		if (result < (snd_pcm_sframes_t)rate->slave->period_size) {
+		result = snd_pcm_mmap_commit(rate->gen.slave, slave_offset, slave_size);
+		if (result < (snd_pcm_sframes_t)slave_size) {
 			if (result < 0)
 				return result;
-			result = snd_pcm_rewind(rate->slave, result);
+			result = snd_pcm_rewind(rate->gen.slave, result);
 			if (result < 0)
 				return result;
 			return 0;
@@ -1083,42 +1024,42 @@ static int snd_pcm_rate_commit_next_period(snd_pcm_t *pcm, snd_pcm_uframes_t app
 				   pcm->format);
 		snd_pcm_areas_copy(rate->pareas, cont,
 				   areas, 0,
-				   pcm->channels, pcm->period_size - cont,
+				   pcm->channels, size - cont,
 				   pcm->format);
 
 		snd_pcm_rate_write_areas1(pcm, rate->pareas, 0, rate->sareas, 0);
 
 		/* ok, commit first fragment */
-		result = snd_pcm_mmap_begin(rate->slave, &slave_areas, &slave_offset, &slave_frames);
+		result = snd_pcm_mmap_begin(rate->gen.slave, &slave_areas, &slave_offset, &slave_frames);
 		if (result < 0)
 			return result;
 	      __partial:
 		xfer = 0;
-		cont = rate->slave->buffer_size - slave_offset;
-		if (cont > rate->slave->period_size)
-			cont = rate->slave->period_size;
+		cont = rate->gen.slave->buffer_size - slave_offset;
+		if (cont > slave_size)
+			cont = slave_size;
 		snd_pcm_areas_copy(slave_areas, slave_offset,
 				   rate->sareas, 0,
 				   pcm->channels, cont,
-				   rate->slave->format);
-		result = snd_pcm_mmap_commit(rate->slave, slave_offset, cont);
+				   rate->gen.slave->format);
+		result = snd_pcm_mmap_commit(rate->gen.slave, slave_offset, cont);
 		if (result < (snd_pcm_sframes_t)cont) {
 			if (result < 0)
 				return result;
-			result = snd_pcm_rewind(rate->slave, result);
+			result = snd_pcm_rewind(rate->gen.slave, result);
 			if (result < 0)
 				return result;
 			return 0;
 		}
 		xfer = cont;
 
-		if (xfer == rate->slave->period_size)
+		if (xfer == slave_size)
 			return 1;
 		
 		/* commit second fragment */
-		cont = rate->slave->period_size - cont;
+		cont = rate->gen.slave->period_size - cont;
 		slave_frames = cont;
-		result = snd_pcm_mmap_begin(rate->slave, &slave_areas, &slave_offset, &slave_frames);
+		result = snd_pcm_mmap_begin(rate->gen.slave, &slave_areas, &slave_offset, &slave_frames);
 		if (result < 0)
 			return result;
 		if (slave_offset) {
@@ -1128,12 +1069,12 @@ static int snd_pcm_rate_commit_next_period(snd_pcm_t *pcm, snd_pcm_uframes_t app
 		snd_pcm_areas_copy(slave_areas, slave_offset,
 				   rate->sareas, xfer,
 				   pcm->channels, cont,
-				   rate->slave->format);
-		result = snd_pcm_mmap_commit(rate->slave, slave_offset, cont);
+				   rate->gen.slave->format);
+		result = snd_pcm_mmap_commit(rate->gen.slave, slave_offset, cont);
 		if (result < (snd_pcm_sframes_t)cont) {
 			if (result < 0)
 				return result;
-			result = snd_pcm_rewind(rate->slave, result + xfer);
+			result = snd_pcm_rewind(rate->gen.slave, result + xfer);
 			if (result < 0)
 				return result;
 			return 0;
@@ -1142,6 +1083,14 @@ static int snd_pcm_rate_commit_next_period(snd_pcm_t *pcm, snd_pcm_uframes_t app
 	return 1;
 }
 
+static int snd_pcm_rate_commit_next_period(snd_pcm_t *pcm, snd_pcm_uframes_t appl_offset)
+{
+	snd_pcm_rate_t *rate = pcm->private_data;
+
+	return snd_pcm_rate_commit_area(pcm, rate, appl_offset, pcm->period_size,
+					rate->gen.slave->period_size);
+}
+
 static int snd_pcm_rate_grab_next_period(snd_pcm_t *pcm, snd_pcm_uframes_t hw_offset)
 {
 	snd_pcm_rate_t *rate = pcm->private_data;
@@ -1154,54 +1103,54 @@ static int snd_pcm_rate_grab_next_period(snd_pcm_t *pcm, snd_pcm_uframes_t hw_of
 
 	areas = snd_pcm_mmap_areas(pcm);
 	if (cont >= pcm->period_size) {
-		result = snd_pcm_mmap_begin(rate->slave, &slave_areas, &slave_offset, &slave_frames);
+		result = snd_pcm_mmap_begin(rate->gen.slave, &slave_areas, &slave_offset, &slave_frames);
 		if (result < 0)
 			return result;
-		if (slave_frames < rate->slave->period_size)
+		if (slave_frames < rate->gen.slave->period_size)
 			goto __partial;
 		snd_pcm_rate_read_areas1(pcm, areas, hw_offset,
 					 slave_areas, slave_offset);
-		result = snd_pcm_mmap_commit(rate->slave, slave_offset, rate->slave->period_size);
-		if (result < (snd_pcm_sframes_t)rate->slave->period_size) {
+		result = snd_pcm_mmap_commit(rate->gen.slave, slave_offset, rate->gen.slave->period_size);
+		if (result < (snd_pcm_sframes_t)rate->gen.slave->period_size) {
 			if (result < 0)
 				return result;
-			result = snd_pcm_rewind(rate->slave, result);
+			result = snd_pcm_rewind(rate->gen.slave, result);
 			if (result < 0)
 				return result;
 			return 0;
 		}
 	} else {
 		/* ok, grab first fragment */
-		result = snd_pcm_mmap_begin(rate->slave, &slave_areas, &slave_offset, &slave_frames);
+		result = snd_pcm_mmap_begin(rate->gen.slave, &slave_areas, &slave_offset, &slave_frames);
 		if (result < 0)
 			return result;
 	      __partial:
 		xfer = 0;
-		cont = rate->slave->buffer_size - slave_offset;
-		if (cont > rate->slave->period_size)
-			cont = rate->slave->period_size;
+		cont = rate->gen.slave->buffer_size - slave_offset;
+		if (cont > rate->gen.slave->period_size)
+			cont = rate->gen.slave->period_size;
 		snd_pcm_areas_copy(rate->sareas, 0,
 				   slave_areas, slave_offset,
 				   pcm->channels, cont,
-				   rate->slave->format);
-		result = snd_pcm_mmap_commit(rate->slave, slave_offset, cont);
+				   rate->gen.slave->format);
+		result = snd_pcm_mmap_commit(rate->gen.slave, slave_offset, cont);
 		if (result < (snd_pcm_sframes_t)cont) {
 			if (result < 0)
 				return result;
-			result = snd_pcm_rewind(rate->slave, result);
+			result = snd_pcm_rewind(rate->gen.slave, result);
 			if (result < 0)
 				return result;
 			return 0;
 		}
 		xfer = cont;
 
-		if (xfer == rate->slave->period_size)
+		if (xfer == rate->gen.slave->period_size)
 			goto __transfer;
 
 		/* grab second fragment */
-		cont = rate->slave->period_size - cont;
+		cont = rate->gen.slave->period_size - cont;
 		slave_frames = cont;
-		result = snd_pcm_mmap_begin(rate->slave, &slave_areas, &slave_offset, &slave_frames);
+		result = snd_pcm_mmap_begin(rate->gen.slave, &slave_areas, &slave_offset, &slave_frames);
 		if (result < 0)
 			return result;
 		if (slave_offset) {
@@ -1211,12 +1160,12 @@ static int snd_pcm_rate_grab_next_period(snd_pcm_t *pcm, snd_pcm_uframes_t hw_of
 		snd_pcm_areas_copy(rate->sareas, xfer,
 		                   slave_areas, slave_offset,
 				   pcm->channels, cont,
-				   rate->slave->format);
-		result = snd_pcm_mmap_commit(rate->slave, slave_offset, cont);
+				   rate->gen.slave->format);
+		result = snd_pcm_mmap_commit(rate->gen.slave, slave_offset, cont);
 		if (result < (snd_pcm_sframes_t)cont) {
 			if (result < 0)
 				return result;
-			result = snd_pcm_rewind(rate->slave, result + xfer);
+			result = snd_pcm_rewind(rate->gen.slave, result + xfer);
 			if (result < 0)
 				return result;
 			return 0;
@@ -1244,88 +1193,62 @@ static int snd_pcm_rate_grab_next_period(snd_pcm_t *pcm, snd_pcm_uframes_t hw_of
 	return 1;
 }
 
+static int snd_pcm_rate_sync_playback_area(snd_pcm_t *pcm, snd_pcm_uframes_t appl_ptr)
+{
+	snd_pcm_rate_t *rate = pcm->private_data;
+	snd_pcm_t *slave = rate->gen.slave;
+	snd_pcm_uframes_t xfer;
+	snd_pcm_sframes_t slave_size;
+	int err;
+
+	slave_size = snd_pcm_avail_update(slave);
+	if (slave_size < 0)
+		return slave_size;
+
+	if (appl_ptr < rate->last_commit_ptr)
+		xfer = appl_ptr - rate->last_commit_ptr + pcm->boundary;
+	else
+		xfer = appl_ptr - rate->last_commit_ptr;
+	while (xfer >= pcm->period_size &&
+	       (snd_pcm_uframes_t)slave_size >= rate->gen.slave->period_size) {
+		err = snd_pcm_rate_commit_next_period(pcm, rate->last_commit_ptr % pcm->buffer_size);
+		if (err == 0)
+			break;
+		if (err < 0)
+			return err;
+		xfer -= pcm->period_size;
+		slave_size -= rate->gen.slave->period_size;
+		rate->last_commit_ptr += pcm->period_size;
+		if (rate->last_commit_ptr >= pcm->boundary)
+			rate->last_commit_ptr = 0;
+	}
+	return 0;
+}
+
 static snd_pcm_sframes_t snd_pcm_rate_mmap_commit(snd_pcm_t *pcm,
 						  snd_pcm_uframes_t offset ATTRIBUTE_UNUSED,
 						  snd_pcm_uframes_t size)
 {
 	snd_pcm_rate_t *rate = pcm->private_data;
-	snd_pcm_t *slave = rate->slave;
-	snd_pcm_uframes_t appl_offset, xfer;
-	snd_pcm_sframes_t slave_size;
 	int err;
 
 	if (size == 0)
 		return 0;
-	if (pcm->stream == SND_PCM_STREAM_CAPTURE) {
-		snd_atomic_write_begin(&rate->watom);
-		snd_pcm_mmap_appl_forward(pcm, size);
-		snd_atomic_write_end(&rate->watom);
-		return size;
-	}
-	slave_size = snd_pcm_avail_update(slave);
-	if (slave_size < 0)
-		return slave_size;
-	xfer = rate->appl_ptr % pcm->period_size;
-	appl_offset = (rate->appl_ptr - xfer) % pcm->buffer_size;
-	xfer = pcm->period_size - xfer;
-	if (xfer >= size) {
-		if (xfer == size && (snd_pcm_uframes_t)slave_size >= rate->slave->period_size) {
-			err = snd_pcm_rate_commit_next_period(pcm, appl_offset);
-			if (err < 0)
-				return err;
-			if (err == 0)
-				return 0;
-		}
-		snd_atomic_write_begin(&rate->watom);
-		snd_pcm_mmap_appl_forward(pcm, size);
-		snd_atomic_write_end(&rate->watom);
-		return size;
-	} else {
-		if ((snd_pcm_uframes_t)slave_size >= rate->slave->period_size) {
-			err = snd_pcm_rate_commit_next_period(pcm, appl_offset);
-			if (err < 0)
-				return err;
-			if (err == 0)
-				return 0;
-		}
-		snd_atomic_write_begin(&rate->watom);
-		snd_pcm_mmap_appl_forward(pcm, xfer);
-		snd_atomic_write_end(&rate->watom);
-		appl_offset += pcm->period_size;
-		appl_offset %= pcm->buffer_size;
-		size -= xfer;
-		slave_size -= rate->slave->period_size;
-	}
-	while ((snd_pcm_uframes_t)size >= pcm->period_size &&
-	       (snd_pcm_uframes_t)slave_size >= rate->slave->period_size) {
-		err = snd_pcm_rate_commit_next_period(pcm, appl_offset);
-		if (err == 0)
-			return xfer;
+	if (pcm->stream == SND_PCM_STREAM_PLAYBACK) {
+		err = snd_pcm_rate_sync_playback_area(pcm, rate->appl_ptr + size);
 		if (err < 0)
-			return xfer > 0 ? (snd_pcm_sframes_t)xfer : err;
-		xfer += pcm->period_size;
-		size -= pcm->period_size;
-		slave_size -= rate->slave->period_size;
-		appl_offset += pcm->period_size;
-		appl_offset %= pcm->buffer_size;
-		snd_atomic_write_begin(&rate->watom);
-		snd_pcm_mmap_appl_forward(pcm, pcm->period_size);
-		snd_atomic_write_end(&rate->watom);
+			return err;
 	}
-	size %= pcm->period_size;
-	if (size > 0) {
-		snd_atomic_write_begin(&rate->watom);
-		snd_pcm_mmap_appl_forward(pcm, size);
-		snd_atomic_write_end(&rate->watom);
-		xfer += size;
-	}
-	return xfer;
+	snd_atomic_write_begin(&rate->watom);
+	snd_pcm_mmap_appl_forward(pcm, size);
+	snd_atomic_write_end(&rate->watom);
+	return size;
 }
 
 static snd_pcm_sframes_t snd_pcm_rate_avail_update(snd_pcm_t *pcm)
 {
 	snd_pcm_rate_t *rate = pcm->private_data;
-	snd_pcm_t *slave = rate->slave;
+	snd_pcm_t *slave = rate->gen.slave;
 	snd_pcm_uframes_t slave_size;
 
 	slave_size = snd_pcm_avail_update(slave);
@@ -1334,6 +1257,7 @@ static snd_pcm_sframes_t snd_pcm_rate_avail_update(snd_pcm_t *pcm)
 	snd_atomic_write_begin(&rate->watom);
 	snd_pcm_rate_sync_hwptr(pcm);
 	snd_atomic_write_end(&rate->watom);
+	snd_pcm_rate_sync_playback_area(pcm, rate->appl_ptr);
 	return snd_pcm_mmap_avail(pcm);
  _capture: {
 	snd_pcm_uframes_t xfer, hw_offset, size;
@@ -1342,7 +1266,7 @@ static snd_pcm_sframes_t snd_pcm_rate_avail_update(snd_pcm_t *pcm)
 	size = pcm->buffer_size - xfer;
 	hw_offset = snd_pcm_mmap_hw_offset(pcm);
 	while (size >= pcm->period_size &&
-	       slave_size >= rate->slave->period_size) {
+	       slave_size >= rate->gen.slave->period_size) {
 		int err = snd_pcm_rate_grab_next_period(pcm, hw_offset);
 		if (err < 0)
 			return err;
@@ -1350,7 +1274,7 @@ static snd_pcm_sframes_t snd_pcm_rate_avail_update(snd_pcm_t *pcm)
 			return (snd_pcm_sframes_t)xfer;
 		xfer += pcm->period_size;
 		size -= pcm->period_size;
-		slave_size -= rate->slave->period_size;
+		slave_size -= rate->gen.slave->period_size;
 		hw_offset += pcm->period_size;
 		hw_offset %= pcm->buffer_size;
 		snd_pcm_mmap_hw_forward(pcm, pcm->period_size);
@@ -1359,14 +1283,33 @@ static snd_pcm_sframes_t snd_pcm_rate_avail_update(snd_pcm_t *pcm)
  }
 }
 
-static int snd_pcm_rate_mmap(snd_pcm_t *pcm ATTRIBUTE_UNUSED)
+static int snd_pcm_rate_poll_revents(snd_pcm_t *pcm, struct pollfd *pfds, unsigned int nfds, unsigned short *revents)
 {
-	return 0;
+	snd_pcm_rate_t *rate = pcm->private_data;
+	if (pcm->stream == SND_PCM_STREAM_PLAYBACK) {
+		/* Try to sync as much as possible */
+		snd_pcm_rate_hwsync(pcm);
+		snd_pcm_rate_sync_playback_area(pcm, rate->appl_ptr);
+	}
+	return snd_pcm_poll_descriptors_revents(rate->gen.slave, pfds, nfds, revents);
 }
 
-static int snd_pcm_rate_munmap(snd_pcm_t *pcm ATTRIBUTE_UNUSED)
+static int snd_pcm_rate_drain(snd_pcm_t *pcm)
 {
-	return 0;
+	snd_pcm_rate_t *rate = pcm->private_data;
+
+	if (pcm->stream == SND_PCM_STREAM_PLAYBACK) {
+		/* commit the remaining fraction (if any) */
+		snd_pcm_uframes_t size, slave_size;
+		size = rate->appl_ptr - rate->last_commit_ptr;
+		if (size > 0) {
+			slave_size = snd_pcm_rate_slave_frames(pcm, size);
+			if (slave_size > 0)
+				snd_pcm_rate_commit_area(pcm, rate, rate->last_commit_ptr % pcm->buffer_size,
+							 size, slave_size);
+		}
+	}
+	return snd_pcm_drain(rate->gen.slave);
 }
 
 static int snd_pcm_rate_status(snd_pcm_t *pcm, snd_pcm_status_t * status)
@@ -1377,7 +1320,7 @@ static int snd_pcm_rate_status(snd_pcm_t *pcm, snd_pcm_status_t * status)
 	snd_atomic_read_init(&ratom, &rate->watom);
  _again:
 	snd_atomic_read_begin(&ratom);
-	err = snd_pcm_status(rate->slave, status);
+	err = snd_pcm_status(rate->gen.slave, status);
 	if (err < 0) {
 		snd_atomic_read_ok(&ratom);
 		return err;
@@ -1415,46 +1358,47 @@ static void snd_pcm_rate_dump(snd_pcm_t *pcm, snd_output_t *out)
 		snd_pcm_dump_setup(pcm, out);
 	}
 	snd_output_printf(out, "Slave: ");
-	snd_pcm_dump(rate->slave, out);
+	snd_pcm_dump(rate->gen.slave, out);
 }
 
 static snd_pcm_fast_ops_t snd_pcm_rate_fast_ops = {
 	.status = snd_pcm_rate_status,
-	.state = snd_pcm_rate_state,
+	.state = snd_pcm_generic_state,
 	.hwsync = snd_pcm_rate_hwsync,
 	.delay = snd_pcm_rate_delay,
 	.prepare = snd_pcm_rate_prepare,
 	.reset = snd_pcm_rate_reset,
-	.start = snd_pcm_rate_start,
-	.drop = snd_pcm_rate_drop,
+	.start = snd_pcm_generic_start,
+	.drop = snd_pcm_generic_drop,
 	.drain = snd_pcm_rate_drain,
-	.pause = snd_pcm_rate_pause,
+	.pause = snd_pcm_generic_pause,
 	.rewind = snd_pcm_rate_rewind,
 	.forward = snd_pcm_rate_forward,
-	.resume = snd_pcm_rate_resume,
-	.poll_ask = snd_pcm_rate_poll_ask,
+	.resume = snd_pcm_generic_resume,
 	.writei = snd_pcm_mmap_writei,
 	.writen = snd_pcm_mmap_writen,
 	.readi = snd_pcm_mmap_readi,
 	.readn = snd_pcm_mmap_readn,
 	.avail_update = snd_pcm_rate_avail_update,
 	.mmap_commit = snd_pcm_rate_mmap_commit,
+	.poll_descriptors_count = snd_pcm_generic_poll_descriptors_count,
+	.poll_descriptors = snd_pcm_rate_poll_descriptors,
+	.poll_revents = snd_pcm_rate_poll_revents,
 };
 
 static snd_pcm_ops_t snd_pcm_rate_ops = {
-	.close = snd_pcm_rate_close,
-	.info = snd_pcm_rate_info,
+	.close = snd_pcm_generic_close,
+	.info = snd_pcm_generic_info,
 	.hw_refine = snd_pcm_rate_hw_refine,
 	.hw_params = snd_pcm_rate_hw_params,
 	.hw_free = snd_pcm_rate_hw_free,
 	.sw_params = snd_pcm_rate_sw_params,
 	.channel_info = snd_pcm_rate_channel_info,
 	.dump = snd_pcm_rate_dump,
-	.nonblock = snd_pcm_rate_nonblock,
-	.async = snd_pcm_rate_async,
-	.poll_revents = snd_pcm_rate_poll_revents,
-	.mmap = snd_pcm_rate_mmap,
-	.munmap = snd_pcm_rate_munmap,
+	.nonblock = snd_pcm_generic_nonblock,
+	.async = snd_pcm_generic_async,
+	.mmap = snd_pcm_generic_mmap,
+	.munmap = snd_pcm_generic_munmap,
 };
 
 
@@ -1484,8 +1428,8 @@ int snd_pcm_rate_open(snd_pcm_t **pcmp, const char *name, snd_pcm_format_t sform
 	if (!rate) {
 		return -ENOMEM;
 	}
-	rate->slave = slave;
-	rate->close_slave = close_slave;
+	rate->gen.slave = slave;
+	rate->gen.close_slave = close_slave;
 	rate->type = RATE_TYPE_LINEAR;
 	rate->srate = srate;
 	rate->sformat = sformat;
