@@ -45,6 +45,7 @@ extern const char *_snd_module_pcm_dsnoop;
 extern const char *_snd_module_pcm_dshare;
 extern const char *_snd_module_pcm_asym;
 extern const char *_snd_module_pcm_iec958;
+extern const char *_snd_module_pcm_softvol;
 
 static const char **snd_pcm_open_objects[] = {
 	&_snd_module_pcm_adpcm,
@@ -69,7 +70,8 @@ static const char **snd_pcm_open_objects[] = {
 	&_snd_module_pcm_dsnoop,
 	&_snd_module_pcm_dshare,
 	&_snd_module_pcm_asym,
-	&_snd_module_pcm_iec958
+	&_snd_module_pcm_iec958,
+	&_snd_module_pcm_softvol
 };
 	
 void *snd_pcm_open_symbols(void)
