@@ -37,11 +37,13 @@ const char *_snd_module_rawmidi_hw = "";
 #define SNDRV_FILE_RAWMIDI		"/dev/snd/midiC%iD%i"
 #define SNDRV_RAWMIDI_VERSION_MAX	SNDRV_PROTOCOL_VERSION(2, 0, 0)
 
+#ifndef DOC_HIDDEN
 typedef struct {
 	int open;
 	int fd;
 	int card, device, subdevice;
 } snd_rawmidi_hw_t;
+#endif
 
 static int snd_rawmidi_hw_close(snd_rawmidi_t *rmidi)
 {
@@ -153,15 +155,15 @@ static ssize_t snd_rawmidi_hw_read(snd_rawmidi_t *rmidi, void *buffer, size_t si
 }
 
 snd_rawmidi_ops_t snd_rawmidi_hw_ops = {
-	close: snd_rawmidi_hw_close,
-	nonblock: snd_rawmidi_hw_nonblock,
-	info: snd_rawmidi_hw_info,
-	params: snd_rawmidi_hw_params,
-	status: snd_rawmidi_hw_status,
-	drop: snd_rawmidi_hw_drop,
-	drain: snd_rawmidi_hw_drain,
-	write: snd_rawmidi_hw_write,
-	read: snd_rawmidi_hw_read,
+	.close = snd_rawmidi_hw_close,
+	.nonblock = snd_rawmidi_hw_nonblock,
+	.info = snd_rawmidi_hw_info,
+	.params = snd_rawmidi_hw_params,
+	.status = snd_rawmidi_hw_status,
+	.drop = snd_rawmidi_hw_drop,
+	.drain = snd_rawmidi_hw_drain,
+	.write = snd_rawmidi_hw_write,
+	.read = snd_rawmidi_hw_read,
 };
 
 
@@ -220,9 +222,11 @@ int snd_rawmidi_hw_open(snd_rawmidi_t **inputp, snd_rawmidi_t **outputp,
 
 	assert(!(mode & ~(SND_RAWMIDI_APPEND|SND_RAWMIDI_NONBLOCK|SND_RAWMIDI_SYNC)));
 
-	if ((fd = open(filename, fmode)) < 0) {
+	fd = snd_open_device(filename, fmode);
+	if (fd < 0) {
 		snd_card_load(card);
-		if ((fd = open(filename, fmode)) < 0) {
+		fd = snd_open_device(filename, fmode);
+		if (fd < 0) {
 			snd_ctl_close(ctl);
 			SYSERR("open %s failed", filename);
 			return -errno;
