@@ -238,6 +238,10 @@ int snd_timer_close(snd_timer_t *timer)
 {
 	int err;
   	assert(timer);
+	while (!list_empty(&timer->async_handlers)) {
+		snd_async_handler_t *h = list_entry(timer->async_handlers.next, snd_async_handler_t, hlist);
+		snd_async_del_handler(h);
+	}
 	if ((err = timer->ops->close(timer)) < 0)
 		return err;
 	if (timer->name)
@@ -273,6 +277,55 @@ snd_timer_type_t snd_timer_type(snd_timer_t *timer)
 	return timer->type;
 }
 
+/**
+ * \brief Add an async handler for a timer
+ * \param handler Returned handler handle
+ * \param timer timer handle
+ * \param callback Callback function
+ * \param private_data Callback private data
+ * \return 0 otherwise a negative error code on failure
+ *
+ * The asynchronous callback is called when new timer event occurs.
+ */
+int snd_async_add_timer_handler(snd_async_handler_t **handler, snd_timer_t *timer,
+				snd_async_callback_t callback, void *private_data)
+{
+	int err;
+	int was_empty;
+	snd_async_handler_t *h;
+	err = snd_async_add_handler(&h, timer->poll_fd,
+				    callback, private_data);
+	if (err < 0)
+		return err;
+	h->type = SND_ASYNC_HANDLER_TIMER;
+	h->u.timer = timer;
+	was_empty = list_empty(&timer->async_handlers);
+	list_add_tail(&h->hlist, &timer->async_handlers);
+	if (was_empty) {
+		err = snd_timer_async(timer, snd_async_handler_get_signo(h), getpid());
+		if (err < 0) {
+			snd_async_del_handler(h);
+			return err;
+		}
+	}
+	*handler = h;
+	return 0;
+}
+
+/**
+ * \brief Return timer handle related to an async handler
+ * \param handler Async handler handle
+ * \return timer handle
+ */
+snd_timer_t *snd_async_handler_get_timer(snd_async_handler_t *handler)
+{
+	if (handler->type != SND_ASYNC_HANDLER_TIMER) {
+		SNDMSG("invalid handler type %d", handler->type);
+		return NULL;
+	}
+	return handler->u.timer;
+}                                                            
+
 /**
  * \brief get count of poll descriptors for timer handle
  * \param timer timer handle
@@ -383,7 +436,7 @@ size_t snd_timer_info_sizeof()
 
 /**
  * \brief allocate a new snd_timer_info_t structure
- * \param ptr returned pointer
+ * \param info returned pointer
  * \return 0 on success otherwise a negative error code if fails
  *
  * Allocates a new snd_timer_info_t structure using the standard
@@ -502,7 +555,7 @@ size_t snd_timer_params_sizeof()
 
 /**
  * \brief allocate a new snd_timer_params_t structure
- * \param ptr returned pointer
+ * \param params returned pointer
  * \return 0 on success otherwise a negative error code if fails
  *
  * Allocates a new snd_timer_params_t structure using the standard
@@ -544,6 +597,7 @@ void snd_timer_params_copy(snd_timer_params_t *dst, const snd_timer_params_t *sr
 /**
  * \brief set timer auto start
  * \param params pointer to #snd_timer_params_t structure
+ * \param auto_start The boolean value to set
  */
 int snd_timer_params_set_auto_start(snd_timer_params_t * params, int auto_start)
 {
@@ -569,6 +623,7 @@ int snd_timer_params_get_auto_start(snd_timer_params_t * params)
 /**
  * \brief set timer exclusive use
  * \param params pointer to #snd_timer_params_t structure
+ * \param exclusive The boolean value to set
  */
 #ifndef DOXYGEN
 int INTERNAL(snd_timer_params_set_exclusive)(snd_timer_params_t * params, int exclusive)
@@ -604,6 +659,7 @@ use_default_symbol_version(__snd_timer_params_get_exclusive, snd_timer_params_ge
 /**
  * \brief set timer early event
  * \param params pointer to #snd_timer_params_t structure
+ * \param early_event The boolean value to set
  */
 int snd_timer_params_set_early_event(snd_timer_params_t * params, int early_event)
 {
@@ -629,6 +685,7 @@ int snd_timer_params_get_early_event(snd_timer_params_t * params)
 /**
  * \brief set timer ticks
  * \param params pointer to #snd_timer_params_t structure
+ * \param ticks Ticks to set
  */
 void snd_timer_params_set_ticks(snd_timer_params_t * params, long ticks)
 {
@@ -650,6 +707,7 @@ long snd_timer_params_get_ticks(snd_timer_params_t * params)
 /**
  * \brief set timer queue size (32-1024)
  * \param params pointer to #snd_timer_params_t structure
+ * \param queue_size The queue size to set
  */
 void snd_timer_params_set_queue_size(snd_timer_params_t * params, long queue_size)
 {
@@ -671,6 +729,7 @@ long snd_timer_params_get_queue_size(snd_timer_params_t * params)
 /**
  * \brief set timer event filter
  * \param params pointer to #snd_timer_params_t structure
+ * \param filter The event filter bits to set
  */
 #ifndef DOXYGEN
 void INTERNAL(snd_timer_params_set_filter)(snd_timer_params_t * params, unsigned int filter)
@@ -723,7 +782,7 @@ size_t snd_timer_status_sizeof()
 
 /**
  * \brief allocate a new snd_timer_status_t structure
- * \param ptr returned pointer
+ * \param status returned pointer
  * \return 0 on success otherwise a negative error code if fails
  *
  * Allocates a new snd_timer_status_t structure using the standard
@@ -876,7 +935,7 @@ ssize_t snd_timer_read(snd_timer_t *timer, void *buffer, size_t size)
 	assert(timer);
 	assert(((timer->mode & O_ACCMODE) == O_RDONLY) || ((timer->mode & O_ACCMODE) == O_RDWR));
 	assert(buffer || size == 0);
-	return timer->ops->read(timer, buffer, size);
+	return (timer->ops->read)(timer, buffer, size);
 }
 
 /**
